#!/usr/bin/env python
"""
Top-level administrative script for the PRAMS_Mars package.

This program requires either Python version 2.6+ or 3.2+, and is importable as
a python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import sys
import os
import subprocess
import getopt
import shlex
import re
import errno

app_label = "PRAMS_Mars"

#------------------------------------------------------------------------------
def mkdir_p(path):
    """Emulates 'mkdir -p' functionality"""
    try:
        os.makedirs(path)
    except OSError as this_exception:
        if this_exception.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
if "check_output" not in dir(subprocess):
 # subprocess.check_output does not exist (Python 2.6), so use a backported
 #  version from Python 2.7 instead, and "add" it to subprocess:
    def check_output(*popenargs, **kwargs):
        """Run command with arguments and return its output."""

        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            error = subprocess.CalledProcessError(retcode, cmd)
            error.output = output
            raise error
        return output

    subprocess.check_output = check_output
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def main(action):
    """Administrative task driver."""

    #=== Define some useful parameters:

    beg_cwd = os.getcwd()  # Record the current working directory

      # Determine fully-qualified filesystem location of this script:
    anchor_path = os.path.abspath(os.path.dirname(sys.argv[0]))
    os.chdir(os.path.join(anchor_path, ".."))   # Start from a "known" location

      # Frequently referenced parameters:

    local_exe_path = os.path.join(anchor_path, "..", "run", "exe")
    local_runetc_path = os.path.join(anchor_path, "..", "run", "etc")
    local_runegrs_path = os.path.join(anchor_path, "..", "run",
                                      "example-runscripts")
    local_runegnl_path = os.path.join(anchor_path, "..", "run",
                                      "example-namelists")
    local_infra_path = anchor_path
    local_build_path = os.path.join(local_infra_path, "build")

    local_bscript_fpath = os.path.join(local_build_path, "build_script.py")
    version_fpath = os.path.join(local_infra_path, "etc", "version_info")

    bec_other_fpath = os.path.join(local_build_path,
                                   "build_env_config.other_packages")
    other_packages_needed = ["universal_lib", "PRAMS_common"]

      # Further argument processing:

    arg_check_action = ["version", "clean", "build", "install", "query_build"]
    arg_check_clean = ["ALL"]
    arg_check_build = ["serial", "DM_only", "SM_only", "hybrid", "debug"]
    arg_check_unary = ["version", "query_build"]
    if action[0] in arg_check_action:   # Is the specified action valid?
        this_action = action[0]
        len_args_action_list = len(action)
        if this_action in arg_check_unary:   # Should not have extraneous args
            if len_args_action_list > 1:   
                usage()   # Disallow extraneous arguments
                sys.exit(1)
        elif this_action == "install":   # Needs one extra argument
            if len_args_action_list == 2:
                install_image_fpath = os.path.join(beg_cwd, action[1])
            else:
                usage()
                sys.exit(1)
        elif this_action == "clean":   # May have 0 or 1 extra argument
            clean_ALL = False   # Default
            for arg_item in action[1:]:
                if arg_item in arg_check_clean:  # Is the specified token valid?
                    clean_ALL = True
                else:                      # The specified token is invalid
                    usage()
                    sys.exit(1)
        elif this_action == "build":   # May have 0, 1, or 2 extra arguments
            if len_args_action_list > 1:
                for arg_item in action[1:]:
                    if arg_item in arg_check_build:  # Is the token valid?
                        pass
                    else:                      # The specified token is invalid
                        usage()
                        sys.exit(1)
    else:                         # The specified action is invalid
        usage()
        sys.exit(1)

    #=== Perform specified action(s):

      # Read the information file about the location of other needed packages,
      #  parse its contents, and store the result(s):
    top_path_op_d = {}
    with open(bec_other_fpath, "rt") as input_f:
        input_f_raw = input_f.read()
    for pair in shlex.split(input_f_raw, True, True):
        split_pair = (pair.replace(';', ' ')).split('=', 1)
        if split_pair[0].endswith("_tpath"):
            rel_path = os.path.join(local_build_path, split_pair[1].strip())
            abs_path = os.path.abspath(rel_path)
            top_path_op_d[split_pair[0]] = abs_path
        elif split_pair[0].endswith("_admin"):
            if split_pair[1].isspace() or (not split_pair[1]): # No admin_script
                top_path_op_d[split_pair[0]] = [split_pair[1].strip(), False]
            else:   # There is an admin_script to run
                rel_path = os.path.join(local_build_path, split_pair[1].strip())
                abs_path = os.path.abspath(rel_path)
                top_path_op_d[split_pair[0]] = [abs_path, True]
        elif split_pair[0].endswith("_ld_arg"):
            top_path_op_d[split_pair[0]] = split_pair[1].strip()

    if this_action == "version":
          # Read the application information file, parse its contents, and
          #  display the result(s):
        vid = {}
        with open(version_fpath, "rt") as version_f:
            version_f_raw = version_f.read()
        for pair in shlex.split(version_f_raw, True, True):
            split_pair = (pair.replace(';', ' ')).split('=', 1)
            vid[split_pair[0]] = split_pair[1]

        print(app_label+": "+vid["version"].strip())

    elif this_action == "query_build":
          # Simply relay the output of the executed script:
        print(subprocess.check_output([local_bscript_fpath, this_action],
                                      stderr=subprocess.STDOUT,
                                      universal_newlines=True))

    elif this_action == "build":

          # Invoke the admin script(s) for any other needed package(s), and
          #  create dictionary of 'query_build' results:
        qbd ={}
        opn_list = []
        for package in other_packages_needed:
            if top_path_op_d[package+"_ld_arg"]:   # Add an ld argument
                opn_list.append(' '+top_path_op_d[package+"_ld_arg"])
            if top_path_op_d[package+"_admin"][0]:   # Run an admin_script
                cmd_list = [top_path_op_d[package+"_admin"][0], this_action]
                if len_args_action_list > 1:
                    for arg in action[1:]:
                        cmd_list.append(arg)
                subprocess.check_call(cmd_list)   # Build

                cmd_list = [top_path_op_d[package+"_admin"][0], "query_build"]
                package_query_output = subprocess.check_output(cmd_list,
                                                universal_newlines=True) # Query
                for pair in shlex.split(package_query_output, True, True):
                    split_pair = (pair.replace(';', ' ')).split('=', 1)
                    if split_pair[0] in qbd:
                        qbd[split_pair[0]] = qbd[split_pair[0]]+' '+ \
                                             split_pair[1]
                    else:
                        qbd[split_pair[0]] = split_pair[1]

        build_info_admin_fpath = os.path.join(local_build_path,
                                              "build_info_admin")

        text_to_write = \
"other_C_INCLUDES=\""+qbd["C_INCLUDES"]+"\"\n"+ \
"other_Fortran_INCLUDES=\""+qbd["FORTRAN_INCLUDES"]+"\"\n"+ \
"other_Fortran_MODPATHS=\"-I"+qbd["BUILT_MODS_PATH"]+"\"\n"+ \
"other_C_LDFLAGS=\""+qbd["C_LINKER_FLAGS"]+"\"\n"+ \
"other_C_LIBS=\""+qbd["C_LIBS_TO_LINK"]+" -L"+qbd["BUILT_LIBS_PATH"]+" ".join(opn_list)+"\"\n"+ \
"other_Fortran_LDFLAGS=\""+qbd["FORTRAN_LINKER_FLAGS"]+"\"\n"+ \
"other_Fortran_LIBS=\""+qbd["FORTRAN_LIBS_TO_LINK"]+" -L"+qbd["BUILT_LIBS_PATH"]+" ".join(opn_list)+"\"\n"+ \
"FAILED_EXT_LIBS=\""+qbd["FAILED_LIB_CHECKS"]+"\"\n"+ \
"USABLE_EXT_LIBS=\""+qbd["USABLE_LIB_CHECKS"]+"\"\n"
        with open(build_info_admin_fpath, "wt") as text_f:
            text_f.write(text_to_write)

          # List of libraries and/or executables to build:
        libexe_fn_list = ["lib"+app_label+".a",
            os.path.join(local_exe_path, "prams"),
            os.path.join(local_exe_path, "postp"),
            os.path.join(local_exe_path, "PIPP_p-split_A173newoutp"),
            os.path.join(local_exe_path, "PIPP_p-split_A21ncoutp"),
            os.path.join(local_exe_path, "PIPP_p-split_UK_ncoutp"),
            os.path.join(local_exe_path, "PIPP_p-split_LMD_ncoutp"),
            os.path.join(local_exe_path, "PIPP_p-split_A21specoutp"),
            os.path.join(local_exe_path, "PIPP_p-split_A21v23nc"),
            os.path.join(local_exe_path, "PIPP_p-split_A23nc")]

          # Execute the build script as many times as necessary:
        mkdir_p(local_runetc_path)    #
        mkdir_p(local_runegrs_path)   # Ensure these directories exist
        mkdir_p(local_runegnl_path)   #
        for i in range(len(libexe_fn_list)):
            print("Building "+libexe_fn_list[i]+" ...")
              # Ensure the directory for the exe/lib exists:
            mkdir_p(os.path.abspath(os.path.dirname(libexe_fn_list[i])))
            cmd_list = [local_bscript_fpath, this_action, libexe_fn_list[i]]
            if len_args_action_list > 1:
                for arg in action[1:]:
                    cmd_list.append(arg)
            subprocess.check_call(cmd_list)

    elif this_action == "clean":
        if clean_ALL:
              # Execute the admin script ("cleaning" mode) for other packages:
            for package in other_packages_needed:
                if top_path_op_d[package+"_admin"][1]:   # Run an admin_script
                    cmd_list = [top_path_op_d[package+"_admin"][0], this_action]
                    if len_args_action_list > 1:
                        for arg in action[1:]:
                            cmd_list.append(arg)
                    subprocess.check_call(cmd_list)

          # Execute the build script (its "cleaning" mode):
        print("Cleaning "+app_label+" debris ...")
        cmd_list = [local_bscript_fpath, this_action]
        if len_args_action_list > 1:
            for arg in action[1:]:
                cmd_list.append(arg)
        subprocess.check_call(cmd_list)

    elif this_action == "install":
          # Install the specified archive image:

        print("Installing "+app_label+" archive image ...")

        if not (os.path.isfile(install_image_fpath) or \
                os.path.islink(install_image_fpath)):
            print(app_label+" ERROR: Archive image file ("+ \
                  install_image_fpath+" was not found.")
            sys.exit(1)

        os.chdir(os.path.join(anchor_path, "..", ".."))  # Change to install dir

        install_image_bn = os.path.basename(install_image_fpath)
        tail_match = re.search("(\.[^s]*){1,2}$", install_image_bn)
        if tail_match:
            if tail_match.group(0) == ".tar.bz2":
                p1 = subprocess.Popen(["bzip2", "-dc", install_image_fpath], 
                                      stdout=subprocess.PIPE,
                                      universal_newlines=True)
                p2 = subprocess.Popen(["tar", "-xvf", '-'], stdin=p1.stdout,
                                      universal_newlines=True)
                p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
                output, err = p2.communicate()
            elif tail_match.group(0) == ".tar.gz":
                p1 = subprocess.Popen(["gzip", "-dc", install_image_fpath], 
                                      stdout=subprocess.PIPE,
                                      universal_newlines=True)
                p2 = subprocess.Popen(["tar", "-xvf", '-'], stdin=p1.stdout,
                                      universal_newlines=True)
                p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
                output, err = p2.communicate()
            elif tail_match.group(0) == ".tar":
                subprocess.check_call(["tar", "-xvf", install_image_fpath])
            elif tail_match.group(0) == ".zip":
                subprocess.check_call(["unzip", install_image_fpath])
        else:
            print(app_label+" ERROR: Archive image file ("+ \
                  install_image_fpath+" has an unsupported format.")
            sys.exit(1)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    usage_msg = """
usage: admin_script.py [-h]  action

Top-level administrative script for this package.  This program requires either
Python version 2.6+ or 3.2+, and is importable as a python module.

mandatory argument:
  action -- One of the following:
      version  :  output the version ID of this code package
      clean  :  Clean up local non-configure build process debris
      clean ALL  :  Clean up all known build process debris, including
                    that of configure and other linked packages
      build [debug]  :  Build package
       " "  serial [debug]  :  (DEFAULT) all parallel features disabled
       " "  DM_only [debug]  :  distributed-memory parallel (MPI) only
       " "  SM_only [debug]  :  shared-memory parallel (OpenMP) only
       " "  hybrid [debug]  :  hybrid parallel (MPI + OpenMP)
      install FILE  :  Install a specified code package update
      query_build  :  Usage/linking info for the built package

optional arguments:
  -h, --help   show this help message and exit
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'h', ["help"])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    # NONE
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 3 or len(args) < 1):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)
    action = args

    main(action)
