!```````````````````````````````````````````````````````````````````
module body_specific_constants_mod

!+ Frequently used Mars "constants"

use univ_kind_defs_mod, only: f4, f8
use univ_math_constants_mod, only: twopi_f8, pi_f4
use univ_phys_constants_mod, only: R_univ_gas_f4
use univ_convert_units_mod, only: g_to_kg_f4

implicit none

!================================== COMMON ==================================

!%%% Physical values (some are guesses, however):

real(f4), parameter ::  &

     !*** Water substance values ***
     molmass_H2O = 18.0153_f4*g_to_kg_f4,  &  ! Molar mass of H2O [kg/mol]
     molmass_CO2 = 44.01_f4*g_to_kg_f4,  &  ! Molar mass of CO2 [kg/mol]

     !*** CO2 ice substance values ***
       ! CO2 ice params from LMD MGCM documentation (Forget, Hourdin, etc.):
     rho_CO2ice = 1630._f4,  &  ! Density of CO2 ice [kg/m^3]
     albd_CO2ice = 0.5_f4,  &  ! Albedo of CO2 ice (a guess) [none]
     emiss_CO2ice = 0.92_f4,  & ! CO2 ice emissivity (an educated guess) [none]
       ! Better value at very cold temperatures (?; 5.9e5 before):
     lth_CO2ice = 5.95e5_f4,  & ! CO2 latent heat (s<->g) [J/kg]

     !*** Dust substance values ***
     rho_dust = 2700._f4               ! Dust particle mass density [kg/m^3]

!================================ END COMMON ================================


!================================ ATMOSPHERE ================================

!%%% Chemical/physical properties of *contemporary* atmospheric constituents:

real(f4), parameter ::  &

     ! Dry atmosphere: 
     R_air = 192._f4,  & ! Gas constant of Mars air [J/(K.kg)]
     cp = 770._f4,  & ! Heat capacity (const press) [J/(K.kg)]
     cv = 578._f4,  & !  " " (const vol); cv=cp-R  [J/(K.kg)]

     ! Reference values:
     p00 = 1.e3_f4,  & ! Ref pressure (for theta/pi calcs) [Pa]

     ! Water substance values:
     R_H2Ov = R_univ_gas_f4/molmass_H2O,  &  ! H2O vapor gas constant [J/(K.kg)]

     ! Values derived from those above:

     rocp = R_air / cp,  & ! [none]
     cpor = cp / R_air,  & ! [none]
     rocv = R_air / cv,  & ! [none]
     cvor = cv / R_air,  & ! [none]
     p00i = 1._f4 / p00,  & ! [Pa^-1]
     cpi  = 1._f4 / cp,  & ! [(K.kg)/J]
     cv_o_cp = cv / cp,  & ! [none]
     cp_o_cv = cp / cv,  & ! [none]
     ep = R_air / R_H2Ov,  & ! [none]
     Tv_fact = (R_H2Ov-R_air)/R_air    ! Tv ~= T*(1.+Tv_fact*q) [none]

!%%% Non-physical or quasi-physical atmosphere-related constants:

real(f4), parameter ::  &

     ! Turbulence-related values:
     vonk = 0.40_f4,  & ! von Karman constant [none]
     tkmin = 5.e-4_f4                  ! Smallest SGS TKE permitted [m^2/s^2]

!============================== END ATMOSPHERE ==============================


!=================================== BODY ===================================

!%%% Surface/subsurface parameters:

real(f4), parameter ::  &

     !*** Subsurface parameters (constant for now) ***
       ! NASA ARC MGCM value -- range  1000. to 1800.
     s_rho = 1500._f4,  & ! Subsurface density [kg/m^3]
       ! Chris McKay publication -- range 627. to 795.
     s_hc = 711._f4          ! Subsurface specific heat [J/(kg.K)]

!%%% Body parameters:

real(f8), parameter ::  &
     sidldy = 88642.663_f8,  & ! Sidereal rotation period [s] {1,3}
     mass = 6.4185e23_f8             ! Planetary system mass [kg] {1,2}

real(f4), parameter ::  &
     g_Xoid = 3.72_f4,  & ! "sfc" gravitational accel [m/s^2]
     radius = 3.390e6_f4,  & ! Radius of sphere of equal volume [m]
     dist_deglat = radius*pi_f4/180._f4,  & ! Degree of latitude length [m]
     omega = real(twopi_f8/sidldy, kind=f4)    ! Axial angular velocity [rad/s]

!================================= END BODY =================================


!================================== BEYOND ==================================

!%%% Orbit-related parameters:

real(f8), parameter ::  &
     obliq = 25.1919_f8,  & ! Axial obliquity (to orbit) [deg] {3}
     mnsdy = 88775.244_f8,  & ! Length of mean solar day [s] {3}
     AUtoM_kludge = 1.000085_f8,  & ! "Kludge" factor [none] that may be
                                       !      used to "tune" the actual AU->m
                                       !      factor so that 
                                       !      `mean_anom(ntabent-1)` ~= 2*pi
                                       !      in `orbtime_tabinit`
     morbpd = 5.9354e7_f8,  & ! Tropical orbital period [s];
                                       !      actually 5.93544d7 in {3}, but
                                       !      "tuned" so that `orbtab_inc`
                                       !      (module orbtime) is an even
                                       !      divisor
     lonasn = 49.57854_f8,  & ! Longitude of ascending node [deg] {5}
     eccen = 0.09341233_f8,  & ! Orbital eccentricity [none] {5}
     inclin = 1.85061_f8,  & ! Orbital inclination [deg] {5}
     smaxis = 1.52366231_f8,  & ! Orbital semimajor axis [AU] {5}
     lonper = 336.04084_f8,  & ! Longitude of perihelion [deg] {5}
     massratio_Sun2bodysys = 3098708._f8,  & ! Mass ratio (Sun/Mars_sys) [none] {2}
     mass_bodysys = 6.4185e23_f8,  & ! Mass of Mars system [kg] {4}
     lsper = 250.999_f8              ! Ls of perihelion [deg] {3}
 
real(f4), parameter ::  &
     solar_mean_broadband = 1.3661e3_f4 ! Mean broadband solar
                                        ! flux @1AU [W/m^2] {6}

!================================ END BEYOND ================================


!================================ REFERENCES ================================

! {1} http://ssd.jpl.nasa.gov/phys_props_planets.html (circa 08.10.2003)
!     {Yoder, C.F. 1995. "Astrometric and Geodetic Properties of Earth and the
!     Solar System" in Global Earth Physics, A Handbook of Physical Constants,
!     AGU Reference Shelf 1, American Geophysical Union, Tables 6,7,10.}
!
! {2} http://ssd.jpl.nasa.gov/?constants (circa 19.01.2013)
!     {Data are from the 1994 IAU file of current best estimates. Planetary
!     ranging determines the Earth/Moon mass ratio. Reference: Standish, E.M.
!     (1995) ``Report of the IAU WGAS Sub-Group on Numerical Standards'', in
!     Highlights of Astronomy (I. Appenzeller, ed.), Table 1, Kluwer Academic
!     Publishers, Dordrecht.}
!
! {3} from Allison, M., and M. McEwen 2000. A post-Pathfinder evaluation of
!     aerocentric solar coordinates with improved timing recipes for Mars
!     seasonal/diurnal climate studies. Planet. Space Sci. 48, 215-235,
!     doi:10.1016/S0032-0633(99)00092-6.
!     ALSO SEE: Allison, M. 1997. Accurate analytic representations of solar
!     time and seasons on Mars with applications to the Pathfinder/Surveyor
!     missions. Geophys. Res. Lett. 24, 1967-1970.
!     [contains Ls~=("heliocentric longitude"-85.06)]
!
! {4} http://nssdc.gsfc.nasa.gov/planetary/factsheet/marsfact.html
!     (circa 08.10.2003)
!
! {5} http://ssd.jpl.nasa.gov/elem_planets.html (circa 08.10.2003)
!     {This table contains mean orbit solutions from a 250 yr. least squares fit
!     of the DE 200 planetary ephemeris to a Keplerian orbit where each element
!     is allowed to vary linearly with time. This solution fits the terrestrial
!     planet orbits to ~25" or better, but achieves only ~600" for Saturn.
!     Elements are referenced to mean ecliptic and equinox of J2000 at the
!     J2000 epoch (2451545.0 JD).}
!
! {6} Gueymard, C., The sun's total and spectral irradiance for solar energy
!     applications and solar radiation models, Solar Energy, Volume 76,
!     Issue 4, April 2004, Pages 423-453, doi:10.1016/j.solener.2003.08.039

!============================== END REFERENCES ==============================


end module body_specific_constants_mod
