;+ NCL routine to plot PRAMS grid locations (with topo background) using
;   a data file generated using the 'GRID_INFO_FILE' mode of PRAMS

; This visualization script is Mars-specific

;=============== USER MAY WANT/NEED TO MODIFY THESE ==========================

; The maximum topography data file resolution to use (4 = 1/4 degree grid).
;  Typical choices: 4, 16, 32, 64, 128
maximum_topo_iq = 32

; Number and Lat/lon coords of any markers to place on the plot:
n_markers = 1
marker_lat = (/ -84.805 /)
marker_lon = (/ 65.713 /)

; File format of output plot -- choice of: NCGM ("ncgm"; use 'idt' to view), 
;  PostScript ("ps", "eps", or "epsi"), or PDF ("pdf")
plot_file_format = "pdf"

; Name of output plot file (without file format extension):
plot_fn_wo_ext = "grid_display_PRAMS"

; Name of input data file generated using the 'GRID_INFO_FILE' mode of PRAMS:
grid_info_fn = "grid_display-data.nc"

;=============================================================================

load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"

begin

 ; Add path information ('vis_dir' is a mandatory command-line argument):
  plot_fpath_wo_ext = vis_dir+"/"+plot_fn_wo_ext
  grid_info_fpath = vis_dir+"/"+grid_info_fn

;=== Initial data input and processing:

 ; Open "grid info" NetCDF file(s):
  grid_info_f = addfile(grid_info_fpath, "r")

 ; Extract/determine useful dimensions (from file):
  grid_info_dims = getfiledimsizes(grid_info_f)
  n_grids = grid_info_dims(1)
  x_f_dims = grid_info_dims(2:2*n_grids:2) 
  max_x_f_dim = max(x_f_dims)
  y_f_dims = grid_info_dims(3:2*n_grids+1:2)
  max_y_f_dim = max(y_f_dims)
 
 ; Create grid ID (sub)strings:
  grid_ID_str = new((/ n_grids /), string)
  do igd=0,n_grids-1
    grid_ID_str(igd) = sprinti("%2.2i", igd+1)
  end do

 ; Grid-independent data:
  input_sfc_data_dir = grid_info_f@input_sfc_data_dir
  pole_lon = grid_info_f->polelon
  pole_lat = grid_info_f->polelat
  line_colors = (/ "black", "white" /)
  topo_f_available_ppd = (/ 4, 16, 32, 64, 128 /)
  n_entries_topo_f_ppd = 5

 ; Instantiate various grid-dependent arrays:
  lon_f = new((/ n_grids, max_y_f_dim, max_x_f_dim /), float)
  lat_f = new((/ n_grids, max_y_f_dim, max_x_f_dim /), float)
  pt_scale_factor = new((/ n_grids, max_y_f_dim, max_x_f_dim /), float)
  grid_box_lon = new((/ n_grids, 2*(max_y_f_dim+max_x_f_dim)-4+2 /), float)
  grid_box_lat = new((/ n_grids, 2*(max_y_f_dim+max_x_f_dim)-4+2 /), float)

 ; Extract simple grid-dependent arrays(s):
  grid_sp_nom_deg = grid_info_f->grid_sp_nom_deg
  grid_sp_nom_m = grid_info_f->grid_sp_nom_m

 ; Fill/calculate grid-dependent data arrays:
  do igd=0,n_grids-1

   ; Lat/lon of model grid points:
    tmp_str = "lon_g"+grid_ID_str(igd)
    lon_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1) = (/ \
                                                       grid_info_f->$tmp_str$ /)
    tmp_str = "lat_g"+grid_ID_str(igd)
    lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1) = (/ \
                                                       grid_info_f->$tmp_str$ /)

   ; Point scale factor:
    tmp_str = "pt_scale_f_g"+grid_ID_str(igd)
    pt_scale_factor(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1) = (/ \
                                                       grid_info_f->$tmp_str$ /)

   ; Lat/lon of grid boundary points:

    loc_b = 0
    loc_e = loc_b+x_f_dims(igd)-1
    grid_box_lat(igd,loc_b:loc_e) = (/ lat_f(igd,0,0:x_f_dims(igd)-1) /)
    grid_box_lon(igd,loc_b:loc_e) = (/ lon_f(igd,0,0:x_f_dims(igd)-1) /)

    loc_b = loc_e+1
    loc_e = loc_b+y_f_dims(igd)-2
    grid_box_lat(igd,loc_b:loc_e) = (/ \
                                 lat_f(igd,1:y_f_dims(igd)-1,x_f_dims(igd)-1) /)
    grid_box_lon(igd,loc_b:loc_e) = (/ \
                                 lon_f(igd,1:y_f_dims(igd)-1,x_f_dims(igd)-1) /)

    loc_b = loc_e+1
    loc_e = loc_b+x_f_dims(igd)-2
    i = loc_b-1
    do iq=x_f_dims(igd)-2,0,1
      i = i+1
      grid_box_lat(igd,i) = (/ lat_f(igd,y_f_dims(igd)-1,iq) /)
      grid_box_lon(igd,i) = (/ lon_f(igd,y_f_dims(igd)-1,iq) /)
    end do

    loc_b = loc_e+1
    loc_e = loc_b+y_f_dims(igd)-2
    i = loc_b-1
    do iq=y_f_dims(igd)-2,0,1
      i = i+1
      grid_box_lat(igd,i) = (/ lat_f(igd,iq,0) /)
      grid_box_lon(igd,i) = (/ lon_f(igd,iq,0) /)
    end do
   
    grid_box_lat(igd,loc_e+1) = grid_box_lat(igd,0)
    grid_box_lon(igd,loc_e+1) = grid_box_lon(igd,0)
       
  end do

  n_box_points = 2*(y_f_dims+x_f_dims)-4

 ; Open output file and define color table
  wks = gsn_open_wks(plot_file_format, plot_fpath_wo_ext)
  gsn_define_colormap(wks, "rainbow")

  print("Creating "+plot_fpath_wo_ext+"."+plot_file_format+" ...")

;=== Create plots:

 ; Set up plot resource template for all base plots (with map):

  res_base_template = True

  res_base_template@gsnMaximize = True

  res_base_template@mpProjection = "Stereographic"
  res_base_template@mpOutlineOn = False
  res_base_template@mpGridAndLimbOn = True
  res_base_template@mpFillBoundarySets = "NoBoundaries"
  res_base_template@mpGridSpacingF = 10.
  res_base_template@mpCenterLatF = pole_lat
  res_base_template@mpCenterLonF = pole_lon
  res_base_template@mpGreatCircleLinesOn = True ; for "nice" lines over lon=180

  res_base_template@mpLimitMode = "Points"

  res_base_template@gsnFrame = False
  res_base_template@gsnDraw  = False

  do igd_min=0,n_grids-1    ; Loop spanning all grids
    print("Plot starting with grid "+sprinti("%i", (igd_min+1))+" ...")
    grid_box_seg = new((/ n_grids, 2*(max_y_f_dim+max_x_f_dim)-4 /), graphic)
    do igd=igd_min,n_grids-1    ; Loop spanning fewer grids each iteration

      if (igd .eq. igd_min) then  ; Only do for "base" grid of this plot frame

       ; Lat/lon of some selected points of the "base" grid:
        bottom_left = (/ lat_f(igd,0,0), lon_f(igd,0,0) /)
        center_left = (/ lat_f(igd,y_f_dims(igd)/2,0),  \
                         lon_f(igd,y_f_dims(igd)/2,0) /)
        top_left = (/ lat_f(igd,y_f_dims(igd)-1,0),  \
                      lon_f(igd,y_f_dims(igd)-1,0) /)
        max_left = (/ max(lat_f(igd,0:y_f_dims(igd)-1,0)),  \
                      max(lon_f(igd,0:y_f_dims(igd)-1,0)) /)
        min_left = (/ min(lat_f(igd,0:y_f_dims(igd)-1,0)),  \
                      min(lon_f(igd,0:y_f_dims(igd)-1,0)) /)
        bottom_right = (/ lat_f(igd,0,x_f_dims(igd)-1),  \
                          lon_f(igd,0,x_f_dims(igd)-1) /)
        center_right = (/ lat_f(igd,y_f_dims(igd)/2,x_f_dims(igd)-1),  \
                          lon_f(igd,y_f_dims(igd)/2,x_f_dims(igd)-1) /)
        top_right = (/ lat_f(igd,y_f_dims(igd)-1,x_f_dims(igd)-1),  \
                       lon_f(igd,y_f_dims(igd)-1,x_f_dims(igd)-1) /)
        max_right = (/ max(lat_f(igd,0:y_f_dims(igd)-1,x_f_dims(igd)-1)),  \
                       max(lon_f(igd,0:y_f_dims(igd)-1,x_f_dims(igd)-1)) /)
        min_right = (/ min(lat_f(igd,0:y_f_dims(igd)-1,x_f_dims(igd)-1)),  \
                       min(lon_f(igd,0:y_f_dims(igd)-1,x_f_dims(igd)-1)) /)
        top_center = (/ lat_f(igd,y_f_dims(igd)-1,x_f_dims(igd)/2),  \
                        lon_f(igd,y_f_dims(igd)-1,x_f_dims(igd)/2) /)
        max_top = (/ max(lat_f(igd,y_f_dims(igd)-1,0:x_f_dims(igd)-1)),  \
                     max(lon_f(igd,y_f_dims(igd)-1,0:x_f_dims(igd)-1)) /)
        min_top = (/ min(lat_f(igd,y_f_dims(igd)-1,0:x_f_dims(igd)-1)),  \
                     min(lon_f(igd,y_f_dims(igd)-1,0:x_f_dims(igd)-1)) /)
        bottom_center = (/ lat_f(igd,0,x_f_dims(igd)/2),  \
                           lon_f(igd,0,x_f_dims(igd)/2) /)
        max_bottom = (/ max(lat_f(igd,0,0:x_f_dims(igd)-1)),  \
                        max(lon_f(igd,0,0:x_f_dims(igd)-1)) /)
        min_bottom = (/ min(lat_f(igd,0,0:x_f_dims(igd)-1)),  \
                        min(lon_f(igd,0,0:x_f_dims(igd)-1)) /)
        rough_center = (/ lat_f(igd,y_f_dims(igd)/2,x_f_dims(igd)/2),  \
                          lon_f(igd,y_f_dims(igd)/2,x_f_dims(igd)/2) /)

       ; Determine whether "base" grid crosses a pole:
        tmp_max = max(lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1))
        tmp_a_1D = ndtooned(lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1))
        tmp_dims = dimsizes(lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1))
        tmp_ind = ind_resolve(maxind(tmp_a_1D), tmp_dims)
        tmp_scale_f = pt_scale_factor(igd,tmp_ind(0,0),tmp_ind(0,1))
        grid_crosses_N_pole =  \ 
                  90.-tmp_max .lt. 1.01*grid_sp_nom_deg(igd)/tmp_scale_f
        tmp_min = min(lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1))
        tmp_a_1D = ndtooned(lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1))
        tmp_dims = dimsizes(lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1))
        tmp_ind = ind_resolve(minind(tmp_a_1D), tmp_dims)
        tmp_scale_f = pt_scale_factor(igd,tmp_ind(0,0),tmp_ind(0,1))
        grid_crosses_S_pole =  \
                  tmp_min+90. .lt. 1.01*grid_sp_nom_deg(igd)/tmp_scale_f
        delete(tmp_ind)
        delete(tmp_a_1D)

        grid_does_not_cross_pole = .not.  \
                       (grid_crosses_N_pole .or. grid_crosses_S_pole)

       ; Construct/set "base map" plotting resources:

        res_base = res_base_template

         ; Width [deg] of the "margin" surrounding the "base" grid:
        border_width = grid_sp_nom_deg(igd)*5.*(grid_sp_nom_deg(igd)^(-0.5))

        if (grid_does_not_cross_pole) then  ;;;;;;;;;;;;;;;;;;; Not cross-polar
          res_base@gsnAddCyclic = False

         ; Determine the quadrant of projection that the "base" grid resides in:
          lon_quad = -135.-pole_lon+rough_center(1)
          if (lon_quad .lt. -180.) then
            lon_quad = 360.+lon_quad
          else if (lon_quad .gt. 180.) then
            lon_quad = lon_quad-360.
          end if
          end if

          i_tst = 0
          do while(i_tst .ge. 0)  ; Infinite loop

           ; Determine lat/lon plot bounds, based on map projection quadrant:
            if (lon_quad .gt. -180. .and. lon_quad .le. -90.) then
              if (pole_lat .ge. 0.) then
                left_lat = center_left(0)   
                right_lat = center_right(0)
                bottom_lat = min_bottom(0)-border_width
                top_lat = max_top(0)+border_width
                left_lon = min_left(1)-border_width
                right_lon = max_right(1)+border_width
                bottom_lon = bottom_center(1)
                top_lon = top_center(1)
              else   ; South
                bottom_lat = max_bottom(0)+border_width
                top_lat = min_top(0)-border_width
                left_lon = max_left(1)+border_width
                right_lon = min_right(1)-border_width
                left_lat = center_left(0)   
                right_lat = center_right(0)
                bottom_lon = bottom_center(1)
                top_lon = top_center(1)
              end if
            else if (lon_quad .gt. -90. .and. lon_quad .le. 0.) then
              if (pole_lat .ge. 0.) then
                left_lat = max_left(0)+border_width
                right_lat = min_right(0)-border_width
                bottom_lon = min_bottom(1)-border_width
                top_lon = max_top(1)+border_width    
                bottom_lat = bottom_center(0)
                top_lat = top_center(0) 
                left_lon = center_left(1)
                right_lon = center_right(1)
              else  ; South
                left_lat = min_left(0)-border_width
                right_lat = max_right(0)+border_width
                bottom_lon = max_bottom(1)+border_width
                top_lon = min_top(1)-border_width
                bottom_lat = bottom_center(0)
                top_lat = top_center(0) 
                left_lon = center_left(1)
                right_lon = center_right(1)
              end if
            else if (lon_quad .gt. 0. .and. lon_quad .le. 90.) then
              if (pole_lat .ge. 0.) then
                left_lat = center_left(0)
                right_lat = center_right(0)
                bottom_lat = max_bottom(0)+border_width
                top_lat = min_top(0)-border_width
                left_lon = max_left(1)+border_width
                right_lon = min_right(1)-border_width
                bottom_lon = bottom_center(1)
                top_lon = top_center(1)     
              else  ; South
                bottom_lat = min_bottom(0)-border_width
                top_lat = max_top(0)+border_width
                left_lon = min_left(1)-border_width
                right_lon = max_right(1)+border_width
                left_lat = center_left(0)
                right_lat = center_right(0)
                bottom_lon = bottom_center(1)
                top_lon = top_center(1)  
              end if
            else   ; longitude >90. to 180.
              if (pole_lat .ge. 0.) then
                left_lat = min_left(0)-border_width
                right_lat = max_right(0)+border_width
                bottom_lon = max_bottom(1)+border_width
                top_lon = min_top(1)-border_width    
                bottom_lat = bottom_center(0)
                top_lat = top_center(0)
                left_lon = center_left(1)
                right_lon = center_right(1)
              else   ; South
                left_lat = max_left(0)+border_width
                right_lat = min_right(0)-border_width
                bottom_lon = min_bottom(1)-border_width
                top_lon = max_top(1)+border_width    
                bottom_lat = bottom_center(0)
                top_lat = top_center(0)
                left_lon = center_left(1)
                right_lon = center_right(1)
              end if
            end if
            end if
            end if

           ; Check whether current quandrant will suffice; if not, try twice to
           ;  find a suitable value before giving up:
            max_a = max((/max_left(0), max_right(0), max_top(0), max_bottom(0)/))
            max_b = max((/left_lat, right_lat, bottom_lat, top_lat/)) 
            min_a = min((/min_left(0), min_right(0), min_top(0), min_bottom(0)/))
            min_b = min((/left_lat, right_lat, bottom_lat, top_lat/))
            if ((max_a .le. max_b) .and. (min_a .ge. min_b)) then
              break   ; Satisfactory bounds
            else if (i_tst .gt. 7) then
              lon_quad = orig_lon_quad
              break   ; Simply giving up the search
            else if (i_tst .eq. 0) then
              orig_lon_quad = lon_quad
              lon_quad = orig_lon_quad+22.5
            else if (i_tst .eq. 1) then
              lon_quad = orig_lon_quad-22.5
            else if (i_tst .eq. 2) then
              lon_quad = orig_lon_quad+70.
            else if (i_tst .eq. 3) then
              lon_quad = orig_lon_quad-70.
            else if (i_tst .eq. 4) then
              lon_quad = orig_lon_quad+160.
            else if (i_tst .eq. 5) then
              lon_quad = orig_lon_quad-160.
            else if (i_tst .eq. 6) then
              lon_quad = orig_lon_quad+250.
            else if (i_tst .eq. 7) then
              lon_quad = orig_lon_quad-250.
            end if
            end if
            end if
            end if
            end if
            end if
            end if
            end if
            end if
            end if

           ; Bound lon_quad value:
            if (lon_quad .lt. -180.) then
              lon_quad = 360.+lon_quad
            else if (lon_quad .gt. 180.) then
              lon_quad = lon_quad-360.
            end if
            end if

            i_tst = i_tst+1
          end do
        else     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Cross-polar
          res_base@gsnAddCyclic = True  ; Topo data can be assumed cyclic in lon

         ; Determine lat/lon plot boundaries:

          if (grid_crosses_N_pole) then      ; North
            left_lat = min_left(0)-border_width
            right_lat = min_right(0)-border_width
            bottom_lat = min_bottom(0)-border_width
            top_lat = min_top(0)-border_width
          else if (grid_crosses_S_pole) then      ; South
            left_lat = max_left(0)+border_width
            right_lat = max_right(0)+border_width
            bottom_lat = max_bottom(0)+border_width
            top_lat = max_top(0)+border_width
          end if
          end if

          left_lon = center_left(1)
          right_lon = center_right(1)
          bottom_lon = bottom_center(1)
          top_lon = top_center(1)
        end if 

       ; Ensure that lat/lon plot bounds are within acceptable ranges:
        res_base@mpLeftPointLatF = min((/ 90., max((/ -90., left_lat /)) /))
        res_base@mpRightPointLatF = min((/ 90., max((/ -90., right_lat /)) /))
        res_base@mpBottomPointLatF = min((/ 90., max((/ -90., bottom_lat /)) /))
        res_base@mpTopPointLatF = min((/ 90., max((/ -90., top_lat /)) /))

        res_base@mpLeftPointLonF = min((/ 180., max((/ -180., left_lon /)) /))
        res_base@mpRightPointLonF = min((/ 180., max((/ -180., right_lon /)) /))
        res_base@mpBottomPointLonF =  \
                   min((/ 180., max((/ -180., bottom_lon /)) /))
        res_base@mpTopPointLonF = min((/ 180., max((/ -180., top_lon /)) /))

        res_base@tiMainString = grid_info_f@run_moniker+  \
              " (grids "+sprinti("%i", (igd+1))+" to "+  \
              sprinti("%i", n_grids)+")"
        res_base@tiMainFontHeightF = 0.02

        res_base@lbLabelBarOn = True
        res_base@lbLabelAutoStride = True

        res_base@cnFillOn = True               ; turn on contour fill
        res_base@cnLinesOn = False              ; turn off contour lines
        res_base@cnLineLabelsOn = False 
        res_base@gsnSpreadColors = True               ; span full colormap
        res_base@cnMaxLevelCount = 20

       ; Extract "raw" topography data from separate NetCDF file:

         ; Determine which is the "best" topo file to use:
        tmp_f = 4./grid_sp_nom_deg(igd)   ; Target topo_f_ppd (~4*model_res)
        tmp_f = min((/ tmp_f, topo_f_available_ppd(n_entries_topo_f_ppd-1) /))
        tmp_ind = ind(topo_f_available_ppd .ge. tmp_f)
        iq = tmp_ind(0)
        iq_str = sprinti("%i", min((/maximum_topo_iq, topo_f_available_ppd(iq)/)))
        delete(tmp_ind)

         ; Open topo file; get dimensions of the 'topo' variable:
        topo_fn = input_sfc_data_dir+"/Mars/topo-global-MOLA_"+iq_str+".nc"
        topo_f = addfile(topo_fn, "r")
        topo_f_dims = getfilevardimsizes(topo_f, "topo")

         ; Nominal lat/lon boundaries of plot:
        bnd_lat = (/ res_base@mpLeftPointLatF, res_base@mpRightPointLatF,  \
                     res_base@mpBottomPointLatF, res_base@mpTopPointLatF /)
        bnd_lon = (/ res_base@mpLeftPointLonF, res_base@mpRightPointLonF,  \
                     res_base@mpBottomPointLonF, res_base@mpTopPointLonF /)

         ; Determine what subset of the entire topo file to ingest, taking care
         ;  to treat "special" cases:

        if (grid_does_not_cross_pole) then    ; Non-cross-polar case

          tmp_f = min(bnd_lon)-topo_f@lon_bounds_cell_center(0)
          idx_lon_min = max((/ 0,  \
                  floattoint(tmp_f/topo_f@lon_increment)-1 /))
          tmp_f = max(bnd_lon)-topo_f@lon_bounds_cell_center(0)
          idx_lon_max = min((/ topo_f_dims(1)-1,  \
                  floattoint(tmp_f/topo_f@lon_increment)+1 /))

          grid_straddles_lon180 =  \
               abs(idx_lon_max-idx_lon_min+1) .ge. (7*topo_f_dims(1))/13

         ; If grid straddles +/- 180 degrees longitude, recalculate:
          if (grid_straddles_lon180) then
            tmp_bnd = where(bnd_lon .lt. 0., bnd_lon, -180.)
            tmp_f = max(tmp_bnd)-topo_f@lon_bounds_cell_center(0)
            idx_lon_min = max((/ 0,  \
                  floattoint(tmp_f/topo_f@lon_increment)-1 /))
            tmp_f = max(bnd_lon)-topo_f@lon_bounds_cell_center(0)
            idx_lon_max = min((/ topo_f_dims(1)-1,  \
                  floattoint(tmp_f/topo_f@lon_increment)+1 /))
          end if
 
          tmp_f = max(bnd_lat)-topo_f@lat_bounds_cell_center(0)
          idx_lat_max = min((/ topo_f_dims(0)-1,  \
                               floattoint(tmp_f/topo_f@lat_increment)+1 /))
          tmp_f = min(bnd_lat)-topo_f@lat_bounds_cell_center(0)
          idx_lat_min = max((/ 0, floattoint(tmp_f/topo_f@lat_increment)-1 /))

        else   ; Cross-polar case

          grid_straddles_lon180 = False   ; Only for non-cross-polar cases

          idx_lon_min = 0
          idx_lon_max = topo_f_dims(1)-1

          if (grid_crosses_N_pole) then   ; North
            idx_lat_max = topo_f_dims(0)-1
            tmp_f = min(bnd_lat)-topo_f@lat_bounds_cell_center(0)
            idx_lat_min = max((/ 0, floattoint(tmp_f/topo_f@lat_increment)-1 /))
          else   ; South
            idx_lat_min = 0
            tmp_f = max(bnd_lat)-topo_f@lat_bounds_cell_center(0)
            idx_lat_max = min((/ topo_f_dims(0)-1,  \
                                 floattoint(tmp_f/topo_f@lat_increment)+1 /))
          end if

        end if

         ; Read in topography data subset; calculate associated lat/lon
         ;  coordinates:        
        n_lat_fi = idx_lat_max-idx_lat_min+1
        if (grid_straddles_lon180) then
          n_lon_fi = (idx_lon_max-0+1)+(topo_f_dims(1)-1-idx_lon_min+1)
          topography = new((/ n_lat_fi, n_lon_fi /), short)
          lon_topo = new((/ n_lat_fi, n_lon_fi /), float)
          n_tmp = topo_f_dims(1)-1-idx_lon_min+1
          topography(0:n_lat_fi-1,0:n_tmp-1) =  \
            topo_f->topo(idx_lat_min:idx_lat_max,idx_lon_min:topo_f_dims(1)-1)
          lon_topo(0,0:n_tmp-1) = fspan(  \
           idx_lon_min*topo_f@lon_increment+topo_f@lon_bounds_cell_center(0),  \
             topo_f@lon_bounds_cell_center(1),  \
             n_tmp)  
          topography(0:n_lat_fi-1,n_tmp:n_lon_fi-1) =  \
             topo_f->topo(idx_lat_min:idx_lat_max,0:idx_lon_max)
          lon_topo(0,n_tmp:n_lon_fi-1) = fspan(  \
             topo_f@lon_bounds_cell_center(0),  \
           idx_lon_max*topo_f@lon_increment+topo_f@lon_bounds_cell_center(0),  \
             n_lon_fi-1-n_tmp+1)        
        else
          n_lon_fi = idx_lon_max-idx_lon_min+1
          topography = new((/ n_lat_fi, n_lon_fi /), short)
          lon_topo = new((/ n_lat_fi, n_lon_fi /), float)
          topography(0:n_lat_fi-1,0:n_lon_fi-1) =  \
             topo_f->topo(idx_lat_min:idx_lat_max,idx_lon_min:idx_lon_max)
          lon_topo(0,:) = fspan(  \
           idx_lon_min*topo_f@lon_increment+topo_f@lon_bounds_cell_center(0),  \
           idx_lon_max*topo_f@lon_increment+topo_f@lon_bounds_cell_center(0),  \
             n_lon_fi)
        end if
        do i=1,n_lat_fi-1
          lon_topo(i,:) = lon_topo(0,:)
        end do
        lat_topo = new((/ n_lat_fi, n_lon_fi /), float)
        lat_topo(:,0) = fspan(  \
           idx_lat_min*topo_f@lat_increment+topo_f@lat_bounds_cell_center(0),  \
           idx_lat_max*topo_f@lat_increment+topo_f@lat_bounds_cell_center(0),  \
           n_lat_fi)  
        do i=1,n_lon_fi-1
          lat_topo(:,i) = lat_topo(:,0)
        end do
        topography@lat2d = lat_topo
        topography@lon2d = lon_topo
        topography@long_name = "topographic height above Xoid"   ; Customize

         ; Create "base" plot for this frame:
        base_plot = gsn_csm_contour_map(wks, topography, res_base)

         ; Destroy variables that may need to be instantiated differently in
         ;  the next pass:
        delete(topography)
        delete(lon_topo)
        delete(lat_topo)

      end if    ; Finished with "base plot" for this frame

     ; Plot relevant grid bounding boxes:

      res_box = True
      res_box@gsLineThicknessF = 4.

      do i=0,n_box_points(igd)-1
        res_box@gsLineColor = line_colors(i%2)
        grid_box_seg(igd,i) = gsn_add_polyline(wks, base_plot,  \
                    grid_box_lon(igd,i:i+1), grid_box_lat(igd,i:i+1),  \
                    res_box)
      end do

    end do

   ; Plot user-defined "place" marker(s):

    res_mark = True
    res_mark@gsMarkerSizeF = 0.007*1.
    res_mark@gsMarkerIndex = 16
    res_mark@gsMarkerColor = "black"

    marker_plot = gsn_add_polymarker(wks, base_plot, marker_lon, marker_lat,  \
                                     res_mark)

   ; Draw/finalize this plot frame:
    draw(base_plot)
    frame(wks)

   ; Special "actual" gridspacing plot (only done once, for the coarsest grid):

    if (igd_min .eq. 0) then

      igd = 0   ; Coarsest grid

     ; Construct/set plotting resources:

      res_dist = res_base_template

      res_dist@mpLeftPointLatF = res_base@mpLeftPointLatF
      res_dist@mpLeftPointLonF = res_base@mpLeftPointLonF
      res_dist@mpRightPointLatF = res_base@mpRightPointLatF 
      res_dist@mpRightPointLonF = res_base@mpRightPointLonF
      res_dist@mpBottomPointLatF = res_base@mpBottomPointLatF
      res_dist@mpBottomPointLonF = res_base@mpBottomPointLonF
      res_dist@mpTopPointLatF = res_base@mpTopPointLatF
      res_dist@mpTopPointLonF = res_base@mpTopPointLonF

      res_dist@tiMainString = "Gridspacing distance variation on grid #1"
      res_dist@tiMainFontHeightF = 0.02

      res_dist@lbLabelBarOn = True
      res_dist@lbLabelAutoStride = True

      res_dist@cnFillOn = True               ; turn on contour fill
      res_dist@cnLinesOn = False              ; turn off contour lines
      res_dist@cnLineLabelsOn = False 
      res_dist@gsnSpreadColors = True               ; span full colormap
      res_dist@cnMaxLevelCount = 15

;      res_dist@gsnAddCyclic = False   

     ; Calculate variable to plot/contour ("actual" model grid spacing in km):
      grid_sp_real_km = grid_sp_nom_m(igd)/1000./  \
                pt_scale_factor(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1)
      grid_sp_real_km@lat2d = lat_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1)
      grid_sp_real_km@lon2d = lon_f(igd,0:y_f_dims(igd)-1,0:x_f_dims(igd)-1)
      grid_sp_real_km@long_name = "actual gridspacing distance"
      grid_sp_real_km@units = "km"

     ; Create plot for this frame:
      dist_plot = gsn_csm_contour_map(wks, grid_sp_real_km, res_dist)

     ; Draw/finalize this plot frame:
      draw(dist_plot)
      frame(wks)

    end if

    delete(grid_box_seg)
    delete(marker_plot)
    delete(base_plot)

  end do

end
