!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
program splitA173newoutp

!+ Puts NASA Ames MGCM (v1.7.3; C-grid; new output format) run data into
!   PIPP format

use univ_kind_defs_mod, only: f4
use univ_std_fileIO_mod, only: find_file_IOunit, determine_IO_reclen
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use PIPP_format_mod, only: lsubs_ts, utc_ts, lat, lon, topo, rec_size, vtype,  &
     dstring, PIPPf_Nmalloc, PIPPf_w_fill, PIPPf_w_data, PIPPf_w_head,  &
     PIPPf_dealloc, sol_ts, PIPP_data
use main_memory_mod, only: undef_value_f4

implicit none

!== Local declarations:

integer :: nsolpf,ntpsl,ntimes
real(f4) :: sptrop, rgas, g, cp, factl, factm, skind, dz, sz, ftmp1, ftmp2,  &
     snap_tsep_bmin
integer :: ni,nj,nj1,nj_1,nk,nk_SBS,iv,i,j,k,n,it,mlevls

real(f4), allocatable, dimension(:) :: sol_Ls    ! [nsolpf+1]
real(f4), allocatable, dimension(:) :: dsigma   ! [nk]
integer, allocatable, dimension(:) :: nzi,vt    ! [nvar]

real(f4), allocatable, dimension(:,:,:,:,:) :: t_m  &   ! [ni,nj_1,nk,ntpsl,nsolpf]
     ,u_m
real(f4), allocatable, dimension(:,:,:,:,:) :: v_m    ! [ni,nj,nk,ntpsl,nsolpf]
real(f4), allocatable, dimension(:,:,:,:,:) ::  &   ! [ni,nj_1,nk_SBS,ntpsl,nsolpf]
     gt_SBS_m
real(f4), allocatable, dimension(:,:,:,:) :: sp_m  &   ! [ni,nj_1,ntpsl,nsolpf]
     ,gt_sfc_m
real(f4), allocatable, dimension(:,:) :: topo_m       ! [ni,nj_1]

real(f4), allocatable, dimension(:,:,:) :: u_a,v_a,p_a,t_a,gpz_a   ! [ni,nj1,nk]

real(f4), allocatable, dimension(:,:,:) :: gt_SBS_a       ! [ni,nj1,nk_SBS]
real(f4), allocatable, dimension(:,:) :: topo_a,gt_sfc_a     ! [ni,nj1]
real(f4), allocatable, dimension(:) :: zlevSBS      ! {nk_SBS+1}

integer :: lastslash, fnwp_len, nvar, itime, IO_reclen
character(len=128) :: dfname, hfname, gradsctlo, PIPPhdr, PIPPdat

integer :: IOunit
logical :: file_exists

character(len=17), parameter :: exe_config_fn = 'auto_gen-prep.nml'

character(len=MAX_FILE_PATH_LEN) :: dfname_wp, hfname_wp, outprfx, desc_str
namelist /prep_args/ dfname_wp, hfname_wp, outprfx, desc_str

!=== Executable statements:

! Read initial "command-line argument" namelist:

IOunit = find_file_IOunit()   ! Find available IO unit

inquire(file=exe_config_fn, exist=file_exists)
if (.NOT.file_exists) call usage

open (IOunit, file=exe_config_fn, status='old', form='formatted')

read (IOunit, nml=prep_args); close (IOunit)

lastslash = index(dfname_wp,'/',.TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(dfname_wp)
dfname = dfname_wp(lastslash+1:fnwp_len)  ! no path
lastslash = index(hfname_wp,'/',.TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(hfname_wp)
hfname = hfname_wp(lastslash+1:fnwp_len)  ! no path

! Create filenames to use later:

write (gradsctlo,'(a,a4)') outprfx(1:len_trim(outprfx)),'.ctl'
write (PIPPhdr,'(a,a4)') outprfx(1:len_trim(outprfx)),'.hdr'
write (PIPPdat,'(a,a4)') outprfx(1:len_trim(outprfx)),'.dat'

! Open header file; read contents:

open(63,file=hfname_wp(1:len_trim(hfname_wp)),status='OLD',form='unformatted')
read (63) nsolpf,ntpsl   !,(#vars 3Dt,3Dv,3Dtsfc,2Dt)
read (63) nk,ni,nj_1,nj,nk_SBS,mlevls
read (63) sptrop,g,rgas,cp

! Allocate arrays needed to read header:
allocate(sol_Ls(nsolpf+1),dsigma(nk),zlevSBS(nk_SBS+1))

read (63) dsigma

do n=1,nsolpf
   do it=1,ntpsl  
      read (63) ftmp1    ! MGCM model time [Mars-hours]
      read (63) ftmp2   ! Ls (lagged in time by one snapshot) 
      if(it == ntpsl) then                  !
         sol_Ls(n+1) = ftmp2                ! 
      elseif(n == 1 .and. it == 1) then     ! Only add to Ls array when needed
         sol_Ls(n) = ftmp2                  !
      endif                                 !
   enddo
enddo

close(63)   ! Close header file

ntimes = nsolpf*ntpsl  ! Number of snapshots per data file

! Set up PIPP variable descriptor arrays:

nvar = 7
allocate(nzi(nvar),vt(nvar))
nzi(1) = nk   ! Temperature [K]
vt(1) = 1     !
nzi(2) = nk  ! Pressure [Pa]
vt(2) = 1    !
nzi(3) = nk   ! U-wind [m/s]
vt(3) = 1     !  
nzi(4) = nk  ! V-wind [m/s]
vt(4) = 1    !
nzi(5) = nk   ! Areopotential height [m]
vt(5) = 1     !
nzi(6) = 1   ! Surface temperature [K] OR -(ice mass) [kg]
vt(6) = 4    !
nzi(7) = nk_SBS  ! Subsurface temperature [K]
vt(7) = 3        !

! These are C-grid specific arguments:
nj1 = nj+1   ! C-grid specific dimension (to allow for pole value calcs)
call PIPPf_Nmalloc(ntimes,ni,nj1,nvar,nzi) ! Allocate PIPP_format arrays

dstring(1) = desc_str(1:len(dstring(1)))     ! Set PIPP description string

! Interpolate Ls for all times in file:

write (*,'(/a)') 'Calculating Ls values for entire dataset ...'
call intp_lsubs_new(nsolpf,ntpsl,ntimes,sol_Ls,lsubs_ts(:,1))

! Allocate memory for MGCM fields:

allocate(t_m(ni,nj_1,nk,ntpsl,nsolpf)  &   ! Arrays for "raw" MGCM values
     ,u_m(ni,nj_1,nk,ntpsl,nsolpf),v_m(ni,nj,nk,ntpsl,nsolpf)  &
     ,gt_SBS_m(ni,nj_1,nk_SBS,ntpsl,nsolpf),sp_m(ni,nj_1,ntpsl,nsolpf)  &
     ,gt_sfc_m(ni,nj_1,ntpsl,nsolpf),topo_m(ni,nj_1))

allocate(u_a(ni,nj1,nk),v_a(ni,nj1,nk)  &     ! Arrays for "adjusted" values
     ,p_a(ni,nj1,nk),t_a(ni,nj1,nk),gpz_a(ni,nj1,nk),gt_sfc_a(ni,nj1)  &
     ,topo_a(ni,nj1),gt_SBS_a(ni,nj1,nK_SBS))

! Read in *ALL* relevant MGCM data from data file:

write (*,'(/a/)') 'Reading entire MGCM dataset ...'
call rdMGCM_A173newoutp(dfname_wp,nk,nk_SBS,ni,nj_1,nj,nsolpf,ntpsl,t_m,u_m  &
     ,v_m,sp_m,gt_sfc_m,gt_SBS_m,topo_m)

! Process each timestep in sequence; write PIPP format datafile: 

IO_reclen = determine_IO_reclen(PIPP_data(1:rec_size(1)))

   ! Open PIPP format datafile: 
open(IOunit, file=PIPPdat(1:len_trim(PIPPdat)), status='unknown',  &
     form='unformatted', access='direct', recl=IO_reclen)

do n=1,nsolpf

   do it=1,ntpsl  ! Loop over number of times per sol that MGCM output is made

      itime = (n-1)*ntpsl+it  ! Number of times from data file beginning

      write (*,'(a,i2,a,i2,a,f7.2,a)')  &          ! Report progress to user:
           'Processing and writing PIPP format: SOL=',n,', TIMESTEP=',it  &
           ,', LS=',lsubs_ts(itime,1),' ...'

      ! Calculate UTC time (Mars-hours):
      
      utc_ts(itime,1) = (24._f4/real(ntpsl, kind=f4))*real(it-1, kind=f4)
      sol_ts(itime,1) = n

      ! Regrid winds (from C-grid to effectively A-grid):

      call regriduvC_new(ni,nj,nj1,nk,u_m(:,:,:,it,n),v_m(:,:,:,it,n),u_a,v_a)

      ! Calculate "pole" point values of temperature, sfc_temp, topo, etc.:

      call pole_calc(ni,nj,nj1,nk,nk_SBS,t_m(:,:,:,it,n),t_a  &
           ,gt_sfc_m(:,:,it,n),gt_sfc_a,topo_m,topo_a,gt_SBS_m(:,:,:,it,n)  &
           ,gt_SBS_a)

      ! Calculate pressures on sigma levels and correct areopotential height:
   
      call siglevs_calcC_new(ni,nj,nj1,nk,mlevls,sp_m(:,:,it,n),p_a,t_a  &
           ,gpz_a,dsigma,sptrop,rgas,g)

      ! Fill PIPP 4D data cube with one timestep's data:
  
      call PIPPf_w_fill(t_a(:,:,:),1)
      call PIPPf_w_fill(p_a(:,:,:),2)
      call PIPPf_w_fill(u_a(:,:,:),3)
      call PIPPf_w_fill(v_a(:,:,:),4)
      call PIPPf_w_fill(gpz_a(:,:,:),5)
      call PIPPf_w_fill(gt_sfc_a(:,:),6)
      call PIPPf_w_fill(gt_SBS_a(:,:,:),7)

      ! Write the timestep's data to PIPP format file:

      call PIPPf_w_data(IOunit, itime)

   enddo

enddo
snap_tsep_bmin = max(utc_ts(2,1)-utc_ts(1,1), utc_ts(2,1)-utc_ts(1,1))*  &
     60._f4  ! [body-minutes]

close(IOunit)     ! Close PIPP format datafile

! Write PIPP format header file:

ftmp1 = 180._f4/real(nj, kind=f4)  ! MGCM delta-latitude 
ftmp2 = 360._f4/real(ni, kind=f4)  ! MGCM delta-longitude 
do j=1,nj1
   do i=1,ni
      topo(i,j,1) = topo_a(i,j)

      ! Actual MGCM is ftmp1*real(j, kind=f4)-90._f4 [j=1,nj]
      lat(i,j,1) =  ftmp1*real(j-1, kind=f4)-90._f4

      lon(i,j,1) = ftmp2*real(i-1, kind=f4)-180._f4
   enddo
enddo

! Create SBS depths:

skind = 0.06_f4
factl = 0.25_f4
factm = 1.2_f4
  ! First layer
dz = factl*skind
zlevSBS(nk_SBS) = -dz*0.5_f4  ! Reverse_order/negate (PRAMS convention)
!^print *,'sthick',1,dz
sz = dz
  ! Other layers
do k=2,nk_SBS
   dz = dz*factm
!^print *,'sthick',k,dz
   zlevSBS(nk_SBS-k+1) = -(sz+dz*0.5_f4)  ! Reverse_order/negate (PRAMS convention)
   sz = sz+dz
enddo
zlevSBS(nk_SBS+1) = 0._f4   ! Surface

do iv=1,nvar
   vtype(iv,1) = vt(iv)
enddo

call PIPPf_w_head(PIPPhdr)

! Open special header file; write SBS depth info:

open(56,file=PIPPhdr(1:len_trim(PIPPhdr))//'.tzl',status='unknown'  &
     ,form='unformatted')
write (56) nk_SBS
write (56) zlevSBS
close(56)

! Write GrADS descriptor file:

open(IOunit, file=gradsctlo(1:len_trim(gradsctlo)), status='unknown',  &
     form='formatted')

lastslash = index(PIPPdat,'/',.TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(PIPPdat)
dfname = PIPPdat(lastslash+1:fnwp_len)  ! no path
write (IOunit, '(a,2x,a)') 'dset', '^'//dfname(1:len_trim(dfname))

write (IOunit, '(a,2x,e13.6)') 'undef', undef_value_f4
write (IOunit, '(a)') 'title MGCM data'
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'xdef', ni, 'linear',  &
     -180._f4, 360._f4/real(ni, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'ydef', nj1, 'linear',  &
     -90._f4, 180._f4/real(nj, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,i1,2x,i1)') 'zdef', nk, 'linear', 1, 1
i = max(1, nint(snap_tsep_bmin))  ! [body-minutes]
if (i <= 9) then
   write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', ntimes,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else if (i <= 99) then
   write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', ntimes,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else
   i = nint(snap_tsep_bmin/60._f4)
   if (i <= 9) then
      write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', ntimes,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   else
      write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', ntimes,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   end if
end if
write (IOunit, '(a,2x,i2)') 'vars', nvar
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'temp', nk, 99, 'temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'p', nk, 99, 'atm pressure [Pa]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'u', nk, 99, 'u-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'v', nk, 99, 'v-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'az', nk, 99, 'areopotential height [m]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_sfc', 0, 99,  &
     'sfc temp or ice mass [K or -kg]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_SBS', nk_SBS, 99,  &
     'subsfc temperature [K]'

write (IOunit, '(a/)') 'endvars'
close(IOunit)

call PIPPf_dealloc()
deallocate(t_m,u_m,v_m,u_a,v_a,p_a,sp_m,gt_sfc_m,gt_SBS_m,topo_m,topo_a  &
     ,gpz_a,t_a,gt_sfc_a,dsigma,nzi,vt,gt_SBS_a)

write (*, '(/,3(a,/))') 'Created the following file (sets): ',  &
     trim(gradsctlo), trim(outprfx)//'.*'

stop '** Normal completion **'


contains


!```````````````````````````````````````````````````````````````````
subroutine usage

!+ Prints program usage information

use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

write (*,'(/a/)')  &
     'USAGE:  Must be started from run_prep, or a similar helper script'
call exception('usage', FATAL)

end subroutine usage


end program splitA173newoutp


!```````````````````````````````````````````````````````````````````
subroutine pole_calc(nlon,nlat,nlat1,nlyr,nSBSlyr,t_m,t_a,gt_sfc_m,gt_sfc_a  &
     ,topo_m,topo_a,gt_SBS_m,gt_SBS_a)

!+ Calculate "pole" point values of temperature, sfc_temp, topo, etc.

use univ_kind_defs_mod, only: f4

implicit none

integer :: nlon,nlat,nlat1,nlyr,nSBSlyr
real(f4), dimension(nlon,nlat-1,nlyr) :: t_m
real(f4), dimension(nlon,nlat1,nlyr) :: t_a
real(f4), dimension(nlon,nlat-1) :: gt_sfc_m,topo_m
real(f4), dimension(nlon,nlat1) :: gt_sfc_a,topo_a
real(f4), dimension(nlon,nlat-1,nSBSlyr) :: gt_SBS_m
real(f4), dimension(nlon,nlat1,nSBSlyr) :: gt_SBS_a

integer :: i,j,k
real(f4) :: sum1,sum2


! Temperature:

do k=1,nlyr

   do i=1,nlon
      
      do j=1,nlat-1    ! Fill t_a(:,2:nlat,:)
         t_a(i,j+1,k) = t_m(i,j,k)
      enddo

      t_a(i,1,k) = t_a(i,2,k)-(t_a(i,3,k)-t_a(i,2,k))      ! S "pole"
      t_a(i,nlat1,k) = t_a(i,nlat,k)+(t_a(i,nlat,k)-t_a(i,nlat-1,k)) ! N "pole"

   enddo

   sum1 = 0._f4
   sum2 = 0._f4
   do i=1,nlon
      sum1 = sum1+t_a(i,1,k) 
      sum2 = sum2+t_a(i,nlat1,k) 
   enddo
   t_a(1:nlon,1,k) = sum1/real(nlon, kind=f4)    ! Use average for South Pole value
   t_a(1:nlon,nlat1,k) = sum2/real(nlon, kind=f4)    ! Use average for North Pole value

enddo

! Sfc temperature:

do i=1,nlon
      
   do j=1,nlat-1    ! Fill gt_a(:,2:nlat)
      gt_sfc_a(i,j+1) = gt_sfc_m(i,j)
   enddo
   
!#   gt_sfc_a(i,1) = gt_sfc_a(i,2)-(gt_sfc_a(i,3)-gt_sfc_a(i,2))    ! S "pole"
!#   gt_sfc_a(i,nlat1) = gt_sfc_a(i,nlat)+  &
!#        (gt_sfc_a(i,nlat)-gt_sfc_a(i,nlat-1))      ! N "pole"

   gt_sfc_a(i,1) = gt_sfc_a(i,2)   ! S "pole"
   gt_sfc_a(i,nlat1) = gt_sfc_a(i,nlat)   ! N "pole"
enddo

sum1 = 0._f4
sum2 = 0._f4
do i=1,nlon
   sum1 = sum1+gt_sfc_a(i,1) 
   sum2 = sum2+gt_sfc_a(i,nlat1) 
enddo
gt_sfc_a(1:nlon,1) = sum1/real(nlon, kind=f4)    ! Use average for South Pole value
gt_sfc_a(1:nlon,nlat1) = sum2/real(nlon, kind=f4)    ! Use average for North Pole value

! Topography:

do i=1,nlon
      
   do j=1,nlat-1    ! Fill topo_a(:,2:nlat)
      topo_a(i,j+1) = topo_m(i,j)
   enddo

   topo_a(i,1) = topo_a(i,2)-(topo_a(i,3)-topo_a(i,2))      ! S "pole"
   topo_a(i,nlat1) = topo_a(i,nlat)+(topo_a(i,nlat)-topo_a(i,nlat-1))! N "pole"

enddo

sum1 = 0._f4
sum2 = 0._f4
do i=1,nlon
   sum1 = sum1+topo_a(i,1) 
   sum2 = sum2+topo_a(i,nlat1) 
enddo
topo_a(1:nlon,1) = sum1/real(nlon, kind=f4)    ! Use average for South Pole value
topo_a(1:nlon,nlat1) = sum2/real(nlon, kind=f4)    ! Use average for North Pole value

! Subsurface temperatures:

do k=1,nSBSlyr

   do i=1,nlon
      
      do j=1,nlat-1    ! Fill gt_SBS_a(:,2:nlat,:)
         ! Reverse order (PRAMS convention):
         gt_SBS_a(i,j+1,k) = gt_SBS_m(i,j,k)
      enddo

!#      gt_SBS_a(i,1,k) = gt_SBS_a(i,2,k)-  &
!#           (gt_SBS_a(i,3,k)-gt_SBS_a(i,2,k))  ! S "pole"
!#      gt_SBS_a(i,nlat1,k) = gt_SBS_a(i,nlat,k)+  &
!#           (gt_SBS_a(i,nlat,k)-gt_SBS_a(i,nlat-1,k)) ! N "pole"

      ! SBPS temperature extrapolates badly, so just clip:
      gt_SBS_a(i,1,k) = gt_SBS_a(i,2,k)     ! S "pole"
      gt_SBS_a(i,nlat1,k) = gt_SBS_a(i,nlat,k)   ! N "pole"

   enddo

   sum1 = 0._f4
   sum2 = 0._f4
   do i=1,nlon
      sum1 = sum1+gt_SBS_a(i,1,k) 
      sum2 = sum2+gt_SBS_a(i,nlat1,k) 
   enddo
   gt_SBS_a(1:nlon,1,k) = sum1/real(nlon, kind=f4)    ! Use average for South Pole value
   gt_SBS_a(1:nlon,nlat1,k) = sum2/real(nlon, kind=f4)  ! Use avg for North Pole value

enddo

end subroutine pole_calc


!```````````````````````````````````````````````````````````````````
subroutine rdMGCM_A173newoutp(dfnm,nk,nk_SBS,ni,nj_1,nj,nsolpf,ntpsl,t_m,u_m  &
     ,v_m,sp_m,gt_sfc_m,gt_SBS_m,topo_m)

!+ Reads data from Ames MGCM output files (v1.7.3; new format by TIM)

use univ_kind_defs_mod, only: f4

implicit none

integer :: nsolpf,ntpsl,nk,nk_SBS,ni,nj_1,nj
real(f4), dimension(ni,nj_1,nk,ntpsl,nsolpf) :: t_m,u_m
real(f4), dimension(ni,nj,nk,ntpsl,nsolpf) :: v_m
real(f4), dimension(ni,nj_1,ntpsl,nsolpf) :: gt_sfc_m,sp_m
real(f4), dimension(ni,nj_1,nk_SBS,ntpsl,nsolpf) :: gt_SBS_m
real(f4), dimension(ni,nj_1) :: topo_m
character(len=128) :: dfnm

real(f4), allocatable, dimension(:,:,:) :: dump3t      ! [ni,nj_1,nk]
real(f4), allocatable, dimension(:,:,:) :: dump3v        ! [ni,nj,nk]
real(f4), allocatable, dimension(:,:) :: dump2t           ! [ni,nj_1]
real(f4), allocatable, dimension(:,:,:) :: dump3tsfc        ! [ni,nj_1,nk_SBS]

integer :: i,j,k,n,it


! Allocate work arrays:

allocate(dump3t(ni,nj_1,nk),dump3v(ni,nj,nk),dump3tsfc(ni,nj_1,nk_SBS)  &
     ,dump2t(ni,nj_1))

! Open MGCM output data file:

open(64,file=dfnm(1:len_trim(dfnm)),status='old',form='unformatted')

! Read MGCM output data file:

do n=1,nsolpf      ! Read each sol from the data file in sequence

   do it=1,ntpsl       ! Number of times per sol that MGCM output is made

      ! Report progress to user:

      write (*,'(a,i2,a,i2,a)') 'Reading SOL=',n,' TIMESTEP=',it,' ...'

      read (64) dump3t
      do k=1,nk
         do j=1,nj_1
            do i=1,ni      
               t_m(i,j,1+nk-k,it,n) = dump3t(i,j,k)   ! Temperature [K]
            enddo
         enddo
      enddo

      read (64) dump3t
      do k=1,nk
         do j=1,nj_1
            do i=1,ni      
               u_m(i,j,1+nk-k,it,n) = dump3t(i,j,k)   ! U wind component [m/s]
            enddo
         enddo
      enddo

      read (64) dump3v
      do k=1,nk
         do j=1,nj
            do i=1,ni      
               v_m(i,j,1+nk-k,it,n) = dump3v(i,j,k)   ! V wind component [m/s]
            enddo
         enddo
      enddo

      read (64) dump2t
      do j=1,nj_1
         do i=1,ni      
            sp_m(i,j,it,n) = dump2t(i,j)           ! Surface pressure [mb]
         enddo
      enddo

      read (64) dump2t
      do j=1,nj_1
         do i=1,ni      
            gt_sfc_m(i,j,it,n) = dump2t(i,j)   ! Sfc temp [K] / ice mass [-kg]
         enddo
      enddo

      read (64) dump2t
      if(n == 1 .and. it == 1) then   ! Time-independent, so only fill once
         do j=1,nj_1
            do i=1,ni      
               topo_m(i,j) = dump2t(i,j)  ! Topo z [m above areoid]
            enddo
         enddo
      endif

      read (64) dump3tsfc
      do k=1,nk_SBS
         do j=1,nj_1
            do i=1,ni      
               gt_SBS_m(i,j,1+nk_SBS-k,it,n) = dump3tsfc(i,j,k)  ! SBS temp [K]
            enddo
         enddo
      enddo

   enddo
enddo

close(64)        ! Close MGCM output data file

! Deallocate work arrays:

deallocate(dump3t,dump3v,dump3tsfc,dump2t)

!**** Temporarily fill water stuff here: ****

allocate(dump3t(ni,nj_1,24),dump2t(ni,nj_1))        ! Allocate work arrays



deallocate(dump3t,dump2t)                   ! Deallocate work arrays

end subroutine rdMGCM_A173newoutp

