!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
program split_A21specoutp

!+ Puts NASA Ames MGCM (v2.1, circa 2011; C-grid; "special" output
!   for S. Rafkin) run data into PIPP format

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_std_fileIO_mod, only: find_file_IOunit, determine_IO_reclen
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_PROC_NAME_LEN
use PIPP_format_mod, only: lsubs_ts, utc_ts, lat, lon, topo, rec_size, vtype,  &
     dstring, PIPPf_Nmalloc, PIPPf_w_fill, PIPPf_w_data, PIPPf_w_head,  &
     PIPPf_dealloc, sol_ts, PIPP_data

implicit none

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'split_A21specoutp'

integer :: nsolpf, ntpsl, ntimes
real(f4) :: sptrop, rgas, g, cp, ftmp1, ftmp2, snap_tsep_bmin
integer :: ni, nj, nj1, nj_1, nk, nk_SBS, iv, i, j, k, n, it, mlevls, a_err

real(f4), parameter :: undef = 1.e30_f4

real(f4), allocatable, dimension(:) :: sol_Ls    ! {nsolpf+1}
real(f4), allocatable, dimension(:) :: dsigma   ! {nk}
integer, allocatable, dimension(:) :: nzi, vt    ! {nvar}

real(f4), allocatable, dimension(:,:,:,:,:) :: &   ! {ni,nj_1,nk,ntpsl,nsolpf}
     t_m, u_m, H2Ovmr_m, H2Oimr_m, dustmr_m, H2Odmr_m
real(f4), allocatable, dimension(:,:,:,:,:) :: v_m    ! {ni,nj,nk,ntpsl,nsolpf}
real(f4), allocatable, dimension(:,:,:,:,:) :: & ! {ni,nj_1,nk_SBS,ntpsl,nsolpf}
     gt_SBS_m
real(f4), allocatable, dimension(:,:,:,:) :: sp_m,  &  ! {ni,nj_1,ntpsl,nsolpf}
     gt_sfc_m, gi_sfc_m, ghi_sfc_m
real(f4), allocatable, dimension(:,:) :: topo_m       ! {ni,nj_1}

real(f4), allocatable, dimension(:,:,:) :: u_a, v_a, p_a, t_a,  & ! {ni,nj1,nk}
     gpz_a, H2Ovmr_a, H2Oimr_a, H2Odmr_a, dustmr_a, rho_a, H2Ovmc_a,  &
     H2Oimc_a, H2Odmc_a, dustmc_a
real(f4), allocatable, dimension(:,:,:) :: gt_SBS_a       ! {ni,nj1,nk_SBS}
real(f4), allocatable, dimension(:,:) :: topo_a, gt_sfc_a,  &     ! {ni,nj1}
     ghi_sfc_a
real(f4), allocatable, dimension(:) :: zlevSBS      ! {nk_SBS+1}
real(f4), allocatable, dimension(:) :: sdepth      ! {nk_SBS}

integer :: lastslash, fnwp_len, nvar, itime
character(len=MAX_FILE_PATH_LEN) :: dfname, df_path, gradsctlo, PIPPhdr, PIPPdat

integer :: IOunit, IO_reclen
logical :: file_exists

character(len=17), parameter :: exe_config_fn = 'auto_gen-prep.nml'

character(len=MAX_FILE_PATH_LEN) :: dfname_wp, outprfx, desc_str
namelist /prep_args/ dfname_wp, outprfx, desc_str

!=== Executable statements:

! Read initial "command-line argument" namelist:

IOunit = find_file_IOunit()   ! Find available IO unit

inquire(file=exe_config_fn, exist=file_exists)
if (.NOT.file_exists) call usage

open (IOunit, file=exe_config_fn, status='old', form='formatted')

read (IOunit, nml=prep_args); close (IOunit)

lastslash = index(dfname_wp,'/',.TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(dfname_wp)
dfname = dfname_wp(lastslash+1:fnwp_len)  ! no path
df_path = '.'; if (lastslash > 0) df_path = dfname_wp(1:lastslash-1)

! Create filenames to use later:

write (gradsctlo,'(a,a4)') outprfx(1:len_trim(outprfx)),'.ctl'
write (PIPPhdr,'(a,a4)') outprfx(1:len_trim(outprfx)),'.hdr'
write (PIPPdat,'(a,a4)') outprfx(1:len_trim(outprfx)),'.dat'

! Open header file; read contents:

! From J. Schaeffer (circa February 2012; a copy of the original information
!  from him is at the end of this source code file):

nsolpf = 10; ntpsl = 16
ni = 60; nj_1 = 35; nj = nj_1+1; nj1 = nj+1; nk = 24; nk_SBS = 24
mlevls = 2*nk+3
rgas = 189.02_f4; g = 3.72_f4; cp = 735.94_f4
sptrop = 8.0e-4_f4  ! [hPa]

! Allocate arrays needed to read header:
allocate(sol_Ls(nsolpf+1), dsigma(nk), sdepth(nk_SBS), zlevSBS(nk_SBS+1),  &
     stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1')

dsigma(:) = (/ 0.0001237_f4, 0.0002186_f4, 0.0003877_f4, 0.0006877_f4,  &  ! 24
     0.0012199_f4, 0.0021622_f4, 0.0038572_f4, 0.0057287_f4, 0.0094287_f4,  &
     0.0155443_f4, 0.0256262_f4, 0.0422609_f4, 0.0696418_f4, 0.1148966_f4,  &
     0.1530633_f4, 0.1572269_f4, 0.1352191_f4, 0.1212412_f4, 0.0718975_f4,  &
     0.0435678_f4, 0.014_f4, 0.008_f4, 0.003_f4, 0.001_f4 /)
! Depth (measured from surface) of subsurface temperature values [m]:
sdepth(:) = (/ 7.4999998E-03_f4, 2.4000000E-02_f4, 4.3800000E-02_f4,  &  ! 24
     6.7560002E-02_f4, 9.6072003E-02_f4, 0.1302864_f4, 0.1713437_f4,  &
     0.2206125_f4, 0.2797349_f4, 0.3506820_f4, 0.4358183_f4, 0.5379820_f4,  &
     0.6605785_f4, 0.8076942_f4, 0.9842331_f4, 1.196080_f4, 1.450296_f4,  &
     1.755355_f4, 2.121426_f4, 2.560711_f4, 3.087854_f4, 3.720425_f4,  &
     4.479510_f4, 5.390412_f4 /)

ntimes = nsolpf*ntpsl  ! Number of snapshots per data file

! Set up PIPP variable descriptor arrays:

nvar = 12
allocate(nzi(nvar), vt(nvar), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2')
nzi(1) = nk   ! Temperature [K]
vt(1) = 1     !
nzi(2) = nk  ! Pressure [Pa]
vt(2) = 1    !
nzi(3) = nk   ! U-wind [m/s]
vt(3) = 1     !  
nzi(4) = nk  ! V-wind [m/s]
vt(4) = 1    !
nzi(5) = nk   ! Areopotential height [m]
vt(5) = 1     !
nzi(6) = nk  ! H2O vapor mass concentration [kg/m^3]
vt(6) = 1    !
nzi(7) = nk   ! Dust aerosol mass concentration [kg/m^3]
vt(7) = 1     !
nzi(8) = nk  ! H2O_ice aerosol mass concentration [kg/m^3]
vt(8) = 1    !
nzi(9) = nk   ! H2O_ice (dust core) aerosol mass concentration [kg/m^3]
vt(9) = 1     !
nzi(10) = nk_SBS  ! Subsurface temperature [K]
vt(10) = 3        !
nzi(11) = 1   ! Surface temperature [K] OR -(CO2 sfc ice mass) [kg/m^2]
vt(11) = 4    !
nzi(12) = 1  ! H2O surface ice mass [kg/m^2]
vt(12) = 4   !

! These are C-grid specific arguments:
call PIPPf_Nmalloc(ntimes, ni, nj1, nvar, nzi) ! Allocate PIPP_format arrays

dstring(1) = desc_str(1:len(dstring(1)))     ! Set PIPP description string

! Allocate memory for MGCM fields:

allocate(t_m(ni,nj_1,nk,ntpsl,nsolpf),  &   ! Arrays for "raw" MGCM values
     u_m(ni,nj_1,nk,ntpsl,nsolpf), v_m(ni,nj,nk,ntpsl,nsolpf),  &
     gt_SBS_m(ni,nj_1,nk_SBS,ntpsl,nsolpf), sp_m(ni,nj_1,ntpsl,nsolpf),  &
     gt_sfc_m(ni,nj_1,ntpsl,nsolpf), H2Ovmr_m(ni,nj_1,nk,ntpsl,nsolpf),  &
     H2Oimr_m(ni,nj_1,nk,ntpsl,nsolpf), dustmr_m(ni,nj_1,nk,ntpsl,nsolpf),  &
     gi_sfc_m(ni,nj_1,ntpsl,nsolpf), ghi_sfc_m(ni,nj_1,ntpsl,nsolpf),  &
     topo_m(ni,nj_1), H2Odmr_m(ni,nj_1,nk,ntpsl,nsolpf), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '3')

allocate(u_a(ni,nj1,nk), v_a(ni,nj1,nk),  &     ! Arrays for "adjusted" values
     p_a(ni,nj1,nk), t_a(ni,nj1,nk), gpz_a(ni,nj1,nk), gt_sfc_a(ni,nj1),  &
     topo_a(ni,nj1), gt_SBS_a(ni,nj1,nK_SBS), H2Ovmr_a(ni,nj1,nk),  &
     H2Oimr_a(ni,nj1,nk), H2Odmr_a(ni,nj1,nk), dustmr_a(ni,nj1,nk),  &
     H2Ovmc_a(ni,nj1,nk), H2Oimc_a(ni,nj1,nk), H2Odmc_a(ni,nj1,nk),  &
     dustmc_a(ni,nj1,nk), ghi_sfc_a(ni,nj1), rho_a(ni,nj1,nk), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '4')

! Read in *ALL* relevant MGCM data from data file:

write (*, '(/,a,/)') 'Reading entire MGCM dataset ...'
call rdMGCM_A21specoutp(dfname_wp)
H2Odmr_m(:,:,:,:,:) = ZERO_f4   ! No data for this...

! Interpolate Ls for all times in file:
write (*, '(/,a)') 'Calculating Ls values for entire dataset ...'
call intp_lsubs_new(nsolpf, ntpsl, ntimes, sol_Ls, lsubs_ts(:,1))

! Process each timestep in sequence; write PIPP format datafile: 

IO_reclen = determine_IO_reclen(PIPP_data(1:rec_size(1)))

! Open PIPP format datafile: 
open(IOunit, file=PIPPdat(1:len_trim(PIPPdat)), status='unknown',  &
     form='unformatted', access='direct', recl=IO_reclen)

do n=1,nsolpf

   do it=1,ntpsl  ! Loop over number of times per sol that MGCM output is made

      itime = (n-1)*ntpsl+it  ! Number of times from data file beginning

      write (*, '(a,i2,a,i2,a,f7.2,a)')  &          ! Report progress to user
           'Processing and writing PIPP format: SOL=', n, ', TIMESTEP=', it,  &
           ', LS=', lsubs_ts(itime,1), ' ...'

      ! Calculate mean UT time (Mars-hours):
      utc_ts(itime,1) = (24._f4/real(ntpsl, kind=f4))*real(it-1, kind=f4)
      sol_ts(itime,1) = n

      ! Regrid winds (from C-grid to effectively A-grid):
      call regriduvC_new(ni, nj, nj1, nk, u_m(:,:,:,it,n), v_m(:,:,:,it,n),  &
           u_a, v_a)

      ! Convert to [K]/[-kg/m^2] representation:
      do j=1,nj_1
         do i=1,ni
            if (gi_sfc_m(i,j,it,n) > 1.e-9_f4)  &
                 gt_sfc_m(i,j,it,n) = -gi_sfc_m(i,j,it,n)
         end do
      end do

      ! Calculate "pole" point values of temperature, sfc_temp, topo, etc.:
      call pole_calc_3d(t_m(:,:,:,it,n), t_a, 1)
      call pole_calc_2d(gt_sfc_m(:,:,it,n), gt_sfc_a, 0)
      call pole_calc_3d(gt_SBS_m(:,:,:,it,n), gt_SBS_a, 0)
      call pole_calc_2d(ghi_sfc_m(:,:,it,n), ghi_sfc_a, 0)
      call pole_calc_3d(H2Ovmr_m(:,:,:,it,n), H2Ovmr_a, 1)
      call pole_calc_3d(H2Oimr_m(:,:,:,it,n), H2Oimr_a, 1)
      call pole_calc_3d(H2Odmr_m(:,:,:,it,n), H2Odmr_a, 1)
      call pole_calc_3d(dustmr_m(:,:,:,it,n), dustmr_a, 1)

      ! Calculate pressures on sigma levels and correct areopotential height:  
      call siglevs_calcC_new(ni, nj, nj1, nk, mlevls, sp_m(:,:,it,n), p_a,  &
           t_a, gpz_a, dsigma, sptrop, rgas, g)

      rho_a(:,:,:) = p_a(:,:,:)/(rgas*t_a(:,:,:))

      ! Convert mass mixing ratios to mass concentrations:
      H2Ovmc_a(:,:,:) = H2Ovmr_a(:,:,:)*rho_a(:,:,:)
      dustmc_a(:,:,:) = dustmr_a(:,:,:)*rho_a(:,:,:)
      H2Oimc_a(:,:,:) = H2Oimr_a(:,:,:)*rho_a(:,:,:)
      H2Odmc_a(:,:,:) = H2Odmr_a(:,:,:)*rho_a(:,:,:)

      ! Fill PIPP 4D data cube with one timestep's data:
  
      call PIPPf_w_fill(t_a(:,:,:),1)
      call PIPPf_w_fill(p_a(:,:,:),2)
      call PIPPf_w_fill(u_a(:,:,:),3)
      call PIPPf_w_fill(v_a(:,:,:),4)
      call PIPPf_w_fill(gpz_a(:,:,:),5)
      call PIPPf_w_fill(H2Ovmc_a(:,:,:),6)
      call PIPPf_w_fill(dustmc_a(:,:,:),7)
      call PIPPf_w_fill(H2Oimc_a(:,:,:),8)
      call PIPPf_w_fill(H2Odmc_a(:,:,:),9)
      call PIPPf_w_fill(gt_SBS_a(:,:,:),10)
      call PIPPf_w_fill(gt_sfc_a(:,:),11)
      call PIPPf_w_fill(ghi_sfc_a(:,:),12)

      ! Write the timestep's data to PIPP-format file:

      call PIPPf_w_data(IOunit, itime)

   end do

end do

snap_tsep_bmin = max(utc_ts(2,1)-utc_ts(1,1), utc_ts(2,1)-utc_ts(1,1))*  &
     60._f4  ! [body-minutes]

call pole_calc_2d(topo_m(:,:), topo_a, 1)

close(IOunit)     ! Close PIPP format datafile

! Write PIPP format header file:

ftmp1 = 180._f4/real(nj, kind=f4)  ! MGCM delta-latitude 
ftmp2 = 360._f4/real(ni, kind=f4)  ! MGCM delta-longitude 
do j=1,nj1
   do i=1,ni
      topo(i,j,1) = topo_a(i,j)

      ! Actual MGCM is ftmp1*real(j)-90. [j=1,nj]
      lat(i,j,1) =  ftmp1*real(j-1, kind=f4)-90._f4

      lon(i,j,1) = ftmp2*real(i-1, kind=f4)-180._f4
   end do
end do

! Create SBS depths; Reverse_order/negate (PRAMS convention)::
zlevSBS(nk_SBS+1) = 0._f4   ! Surface
do k=1,nk_SBS
   zlevSBS(nk_SBS-k+1) = -sdepth(k)
end do

do iv=1,nvar
   vtype(iv,1) = vt(iv)
enddo

call PIPPf_w_head(PIPPhdr)

! Open special header file; write SBS depth info:

open(IOunit, file=PIPPhdr(1:len_trim(PIPPhdr))//'.tzl', status='unknown',  &
     form='unformatted')
write (IOunit) nk_SBS
write (IOunit) zlevSBS
close(IOunit)

! Write GrADS descriptor file:

open(IOunit, file=gradsctlo(1:len_trim(gradsctlo)), status='unknown',  &
     form='formatted')

lastslash = index(PIPPdat, '/', .TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(PIPPdat)
dfname = PIPPdat(lastslash+1:fnwp_len)  ! no path
write (IOunit, '(a,2x,a)') 'dset', '^'//dfname(1:len_trim(dfname))

write (IOunit, '(a,2x,e13.6)') 'undef', undef
write (IOunit, '(a)') 'title MGCM data'
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'xdef', ni, 'linear',   &
     -180._f4, 360._f4/real(ni, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'ydef', nj1, 'linear',  &
     -90._f4, 180._f4/real(nj, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,i1,2x,i1)') 'zdef', nk, 'linear', 1, 1
i = max(1, nint(snap_tsep_bmin))  ! [body-minutes]
if (i <= 9) then
   write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', ntimes,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else if (i <= 99) then
   write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', ntimes,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else
   i = nint(snap_tsep_bmin/60._f4)
   if (i <= 9) then
      write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', ntimes,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   else
      write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', ntimes,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   end if
end if
write (IOunit, '(a,2x,i2)') 'vars', nvar
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'temp', nk, 99, 'temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'p', nk, 99, 'atm pressure [Pa]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'u', nk, 99, 'u-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'v', nk, 99, 'v-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'az', nk, 99, 'areopotential height [m]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Ov_mc', nk, 99,  &
     'H2O_vapor mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'dust_mc', nk, 99,  &
     'dust aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Oi_mc', nk, 99,  &
     'H2O_ice aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Od_mc', nk, 99,  &
     'H2Oi_dust aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_SBS', nk_SBS, 99,  &
     'subsfc temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_sfc', 0, 99,  &
     'sfc temp or ice mass [K or -kg]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'ghi_sfc', 0, 99,  &
     'sfc H2O ice mass [kg/m^2]'

write (IOunit, '(a,/)') 'endvars'
close(IOunit)

call PIPPf_dealloc

deallocate(u_a, v_a, p_a, t_a, gpz_a, gt_sfc_a, topo_a, gt_SBS_a, H2Ovmr_a,  &
     H2Oimr_a, H2Odmr_a, dustmr_a, H2Ovmc_a, H2Oimc_a, H2Odmc_a, dustmc_a,  &
     ghi_sfc_a, rho_a, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '4')

deallocate(t_m, u_m, v_m, gt_SBS_m, sp_m, gt_sfc_m, H2Ovmr_m, H2Oimr_m,  &
     dustmr_m, gi_sfc_m, ghi_sfc_m, topo_m, H2Odmr_m, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '3')

deallocate(nzi, vt, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '2')

deallocate(sol_Ls, dsigma, sdepth, zlevSBS, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

write (*, '(/,3(a,/))') 'Created the following file (sets): ',  &
     trim(gradsctlo), trim(outprfx)//'.*'

stop '** Normal completion **'


contains


!```````````````````````````````````````````````````````````````````
subroutine usage

!+ Prints program usage information

use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

write (*,'(/a/)')  &
     'USAGE:  Must be started from run_prep, or a similar helper script'
call exception('usage', FATAL)

end subroutine usage


!```````````````````````````````````````````````````````````````````
subroutine pole_calc_2d(array_in, array_out, operation_type)

!+ Calculate "pole" point values of temperature, sfc_temp, topo, etcetera; for
!   two-dimensional arrays

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: operation_type
real(f4), dimension(:,:), intent(IN) :: array_in
real(f4), dimension(:,:), intent(OUT) :: array_out

!== Local declarations:

integer :: i, j, n_lon, n_lat_1, n_lat, n_lat1
real :: sum1, sum2

!=== Executable statements:

n_lon = size(array_in, 1)
n_lat_1 = size(array_in, 2); n_lat = n_lat_1+1; n_lat1 = n_lat+1

select case (operation_type)
case(0)     ! Copy

   do i=1,n_lon
      do j=1,n_lat_1    ! Fill array_out(:,2:n_lat)
         array_out(i,j+1) = array_in(i,j)
      end do
   
      array_out(i,1) = array_out(i,2)   ! S "pole"
      array_out(i,n_lat1) = array_out(i,n_lat)   ! N "pole"
   end do

case(1)   ! Extrapolate

   do i=1,n_lon
      do j=1,n_lat_1    ! Fill array_out(:,2:n_lat)
         array_out(i,j+1) = array_in(i,j)
      end do
      
      array_out(i,1) = array_out(i,2)-(array_out(i,3)-array_out(i,2)) ! S "pole"
      array_out(i,n_lat1) = array_out(i,n_lat)+  &
           (array_out(i,n_lat)-array_out(i,n_lat_1))  ! N "pole"
   end do

end select

sum1 = ZERO_f4; sum2 = ZERO_f4
do i=1,n_lon
   sum1 = sum1+array_out(i,1) 
   sum2 = sum2+array_out(i,n_lat1) 
end do
array_out(1:n_lon,1) = sum1/real(n_lon, kind=f4) ! Use avg for South Pole value
array_out(1:n_lon,n_lat1) = sum2/real(n_lon, kind=f4) ! Use avg for NP value

end subroutine pole_calc_2d


!```````````````````````````````````````````````````````````````````
subroutine pole_calc_3d(array_in, array_out, operation_type)

!+ Calculate "pole" point values of temperature, sfc_temp, topo, etcetera; for
!   three-dimensional arrays

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: operation_type
real(f4), dimension(:,:,:), intent(IN) :: array_in
real(f4), dimension(:,:,:), intent(OUT) :: array_out

!== Local declarations:

integer :: i, j, k, n_lon, n_lat_1, n_lat, n_lat1, n_lyr
real :: sum1, sum2

!=== Executable statements:

n_lon = size(array_in, 1); n_lyr = size(array_in, 3)
n_lat_1 = size(array_in, 2); n_lat = n_lat_1+1; n_lat1 = n_lat+1

select case (operation_type)
case(0)

   do k=1,n_lyr
      do i=1,n_lon
         do j=1,n_lat-1    ! Fill array_out(:,2:n_lat,:)
            array_out(i,j+1,k) = array_in(i,j,k)
         end do
   
         array_out(i,1,k) = array_out(i,2,k)   ! S "pole"
         array_out(i,n_lat1,k) = array_out(i,n_lat,k)   ! N "pole"
      end do
   end do

case(1)

   do k=1,n_lyr
      do i=1,n_lon
         do j=1,n_lat-1    ! Fill array_out(:,2:n_lat,:)
            array_out(i,j+1,k) = array_in(i,j,k)
         end do

         array_out(i,1,k) = array_out(i,2,k)-  &
              (array_out(i,3,k)-array_out(i,2,k))      ! S "pole"
         array_out(i,n_lat1,k) = array_out(i,n_lat,k)+  &
              (array_out(i,n_lat,k)-array_out(i,n_lat-1,k)) ! N "pole"
      end do
   end do

end select

do k=1,n_lyr
   sum1 = ZERO_f4; sum2 = ZERO_f4
   do i=1,n_lon
      sum1 = sum1+array_out(i,1,k) 
      sum2 = sum2+array_out(i,n_lat1,k) 
   end do
   array_out(1:n_lon,1,k) = sum1/real(n_lon, kind=f4) ! Use avg for SP value
   array_out(1:n_lon,n_lat1,k) = sum2/real(n_lon, kind=f4)  ! Use avg for NP val
end do

end subroutine pole_calc_3d


!```````````````````````````````````````````````````````````````````
subroutine rdMGCM_A21specoutp(dfnm)

!+ Reads data from NASA Ames MGCM output files (v2.1, circa 2011; 
!   "special" output for S. Rafkin)

use univ_kind_defs_mod, only: f4
use univ_std_fileIO_mod, only: find_file_IOunit
use univ_math_constants_mod, only: ZERO_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: dfnm

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'rdMGCM_A21specoutp'

real, allocatable, dimension(:,:) :: dump2t       ! {nj_1,ni}
real, allocatable, dimension(:,:) :: dump2v       ! {nj_1,ni} (special)

character(len=MAX_FILE_PATH_LEN) :: fn_tmp
real(f4) :: time_tmp, Ls_tmp
integer :: i, j, k, n, it, IOunit, layer_tmp, a_err

!=== Executable statements:

IOunit = find_file_IOunit()

! Allocate work arrays:

allocate(dump2t(nj_1,ni), dump2v(nj_1,ni), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1')

! Read separate topography data (must be in same directory as MGCM output data
!  file):

fn_tmp = trim(df_path)//'/topog_35x60'
open (IOunit, file=trim(fn_tmp), status='old', form='unformatted',  &
     access='stream')
read (IOunit) dump2t
do j=1,nj_1
   do i=1,ni
      topo_m(i,j) = dump2t(j,i)           ! Surface pressure [mb]
   end do
end do
close(IOunit)

! Open MGCM output data file:

open(IOunit, file=trim(dfnm), status='old', form='unformatted',  &
     access='stream')

! Read MGCM output data file:

do n=1,nsolpf      ! Read each sol from the data file in sequence

   do it=1,ntpsl       ! Number of times per sol that MGCM output is made

      ! Report progress to user:

      write (*, '(a,i2,a,i2,a)') 'Reading SOL=', n, ' TIMESTEP=', it, ' ...'

      read (IOunit) time_tmp, Ls_tmp, layer_tmp
      if (it == 1) sol_Ls(n) = Ls_tmp  ! Only add to Ls array when needed

      read (IOunit) dump2t
      do j=1,nj_1
         do i=1,ni
            sp_m(i,j,it,n) = dump2t(j,i)           ! Surface pressure [mb]
         end do
      end do

      read (IOunit) time_tmp, Ls_tmp, layer_tmp
      read (IOunit) dump2t    ! 'tstrat' (not used)

      do k=1,nk
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2t
         do j=1,nj_1
            do i=1,ni      
               t_m(i,j,1+nk-k,it,n) = dump2t(j,i)   ! Temperature [K]
            end do
         end do
      end do

      do k=1,nk
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2t
         do j=1,nj_1
            do i=1,ni      
               u_m(i,j,1+nk-k,it,n) = dump2t(j,i)   ! U wind component [m/s]
            end do
         end do
      end do

      do k=1,nk
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2v
         do j=1,nj_1
            do i=1,ni      
               v_m(i,j,1+nk-k,it,n) = dump2v(j,i)   ! V wind component [m/s]
            end do
         end do
         v_m(i:ni,nj,1+nk-k,it,n) = ZERO_f4
      end do

      do k=1,nk
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2t
         do j=1,nj_1
            do i=1,ni      
               H2Ovmr_m(i,j,1+nk-k,it,n) = dump2t(j,i) ! H2O vapor [kg_v/kg]
            end do
         end do
      end do

      do k=1,nk
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2t
         do j=1,nj_1
            do i=1,ni      
               H2Oimr_m(i,j,1+nk-k,it,n) = dump2t(j,i) ! H2O cloud_ice [kg_i/kg]
            end do
         end do
      end do

      do k=1,nk
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2t
         do j=1,nj_1
            do i=1,ni      
               dustmr_m(i,j,1+nk-k,it,n) = dump2t(j,i) ! Airborne dust [kg_d/kg]
            end do
         end do
      end do

      read (IOunit) time_tmp, Ls_tmp, layer_tmp
      read (IOunit) dump2t
      do j=1,nj_1
         do i=1,ni      
            gt_sfc_m(i,j,it,n) = dump2t(j,i)   ! Sfc temp [K]
         end do
      end do

      do k=1,nk_SBS
         read (IOunit) time_tmp, Ls_tmp, layer_tmp
         read (IOunit) dump2t
         do j=1,nj_1
            do i=1,ni      
               gt_SBS_m(i,j,1+nk_SBS-k,it,n) = dump2t(j,i)  ! SBS temp [K]
            end do
         end do
      end do

      read (IOunit) time_tmp, Ls_tmp, layer_tmp
      read (IOunit) dump2t
      do j=1,nj_1
         do i=1,ni      
            gi_sfc_m(i,j,it,n) = dump2t(j,i)   ! CO2 supersurface ice [kg/m^2]
         end do
      end do

      read (IOunit) time_tmp, Ls_tmp, layer_tmp
      read (IOunit) dump2t
      do j=1,nj_1
         do i=1,ni      
            ghi_sfc_m(i,j,it,n) = dump2t(j,i)   ! H2O supersurface ice [kg/m^2]
         end do
      end do

   end do
end do

close(IOunit)        ! Close MGCM output data file

! Extrapolate final Ls value:
it = nsolpf; sol_Ls(it+1) = sol_Ls(it)+(sol_Ls(it)-sol_Ls(it-1))

! Deallocate work arrays:

deallocate(dump2t, dump2v, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

end subroutine rdMGCM_A21specoutp


end program split_A21specoutp
