!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
program split_LMD_ncoutp

!+ Puts Laboratoire de Météorologie Dynamique (LMD) Mars GCM NetCDF-format
!   output files (c. 2014) into PIPP format

use univ_kind_defs_mod, only: f4, f4_B, i8, i4
use univ_std_fileIO_mod, only: find_file_IOunit, determine_IO_reclen
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_math_constants_mod, only: ZERO_f4
use PIPP_format_mod, only: lsubs_ts, utc_ts, lat, lon, topo, rec_size, vtype,  &
     dstring, PIPPf_Nmalloc, PIPPf_w_fill, PIPPf_w_data, PIPPf_w_head,  &
     PIPPf_dealloc, sol_ts, PIPP_data
use main_memory_mod, only: undef_value_f4
use manage_NetCDF_classic_mod, only: mNc_file_metadata_obj, mNc_recycle_f_md_obj

implicit none

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'split_LMD_ncoutp'

! Declarations intended to be available "program-wide":

real(f4) :: R_air, g, cp_air, snap_tsep_bmin
integer :: ni, nj1, nk, nk_SBS, iv, i, j, i_snap, i_it, i_sol, IO_reclen,  &
     n_times, i_iter, rpw_n_iter, rpw_snap_b, rpw_snap_e, rpw_snapstride

real(f4), allocatable, dimension(:) :: lat0   ! {nj1}
real(f4), allocatable, dimension(:) :: lon0   ! {ni}
integer, allocatable, dimension(:) :: nzi, vt    ! {n_var}

real(f4), allocatable, dimension(:,:,:,:) ::  &  ! {ni,nj1,nk,n_times}
     t_m, u_m, v_m, H2Ovmr_m, dustmr_m, H2Oimr_m, H2Odmr_m, p_m
real(f4), allocatable, dimension(:,:,:,:) ::  &
     gt_SBS_m                ! {ni,nj1,nk_SBS,n_times}
real(f4), allocatable, dimension(:,:,:) :: sp_m,  & ! {ni,nj1,n_times}
     gt_sfc_m, gi_sfc_m, ghi_sfc_m, dust_asfc_m, H2Oi_asfc_m, H2Od_asfc_m

real(f4), allocatable, dimension(:,:,:) :: u_a, v_a, p_a, t_a,  & ! {ni,nj1,nk}
     bpz_a, H2Ovmc_a, dustmc_a, H2Oimc_a, H2Odmc_a, rho_a
real(f4), allocatable, dimension(:,:,:) :: gt_SBS_a       ! {ni,nj1,nk_SBS}
real(f4), allocatable, dimension(:,:) :: gt_sfc_a, sp_a,  &  ! {ni,nj1}
     ghi_sfc_a, dust_asfc_a, H2Oi_asfc_a, H2Od_asfc_a     
real(f4), allocatable, dimension(:) :: zlevSBS      ! {nk_SBS+1}

integer :: lastslash, fnwp_len, n_var
character(len=256) :: dfname, gradsctlo, PIPPhdr, PIPPdat

integer :: IOunit, e
logical :: file_exists

type(mNc_file_metadata_obj) :: file_in

character(len=17), parameter :: exe_config_fn = 'auto_gen-prep.nml'

character(len=MAX_FILE_PATH_LEN) :: dfname_wp, outprfx, desc_str
namelist /prep_args/ dfname_wp, outprfx, desc_str

!=== Executable statements:

! Read initial "command-line argument" namelist:

IOunit = find_file_IOunit()   ! Find available IO unit

inquire(file=exe_config_fn, exist=file_exists)
if (.NOT.file_exists) call usage

open (IOunit, file=exe_config_fn, status='old', form='formatted')

read (IOunit, nml=prep_args); close (IOunit)

lastslash = index(dfname_wp, '/', .TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(dfname_wp)
dfname = dfname_wp(lastslash+1:fnwp_len)  ! no path

! Create filenames to use later:

write (gradsctlo, '(a,a4)') outprfx(1:len_trim(outprfx)), '.ctl'
write (PIPPhdr, '(a,a4)') outprfx(1:len_trim(outprfx)), '.hdr'
write (PIPPdat, '(a,a4)') outprfx(1:len_trim(outprfx)), '.dat'

! Open datafile; read in time-independent metadata:
call rdGCM_LMDnc(phase=0)

! Set up PIPP variable descriptor arrays:

n_var = 15
allocate(nzi(n_var), vt(n_var), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'A')

nzi(1) = nk; vt(1) = 1    ! Temperature [K]
nzi(2) = nk; vt(2) = 1    ! Pressure [Pa]
nzi(3) = nk; vt(3) = 1    ! U-wind [m/s]
nzi(4) = nk; vt(4) = 1    ! V-wind [m/s]
nzi(5) = nk; vt(5) = 1    ! Body-potential height [m]
nzi(6) = nk; vt(6) = 1    ! H2O vapor mass concentration [kg/m^3]
nzi(7) = nk; vt(7) = 1    ! Dust aerosol mass concentration [kg/m^3]
nzi(8) = nk; vt(8) = 1    ! H2O_ice aerosol mass concentration [kg/m^3]
nzi(9) = nk; vt(9) = 1 ! H2O_ice (dust core) aerosol mass concentration [kg/m^3]
nzi(10) = nk_SBS; vt(10) = 3  ! Subsurface temperature [K]
nzi(11) = 1; vt(11) = 4   ! Surface T [K] OR -(CO2 sfc ice mass) [kg/m^2]
nzi(12) = 1; vt(12) = 4   ! H2O surface ice mass [kg/m^2]
nzi(13) = 1; vt(13) = 4   ! dust aerosol surface mass [kg/m^2]
nzi(14) = 1; vt(14) = 4   ! H2O ice aerosol surface mass [kg/m^2]
nzi(15) = 1; vt(15) = 4   ! H2O ice (dust core) aerosol surface mass [kg/m^2]

   ! These are C-grid specific arguments:
call PIPPf_Nmalloc(n_times, ni, nj1, n_var, nzi) ! Allocate PIPP-format arrays

dstring(1) = desc_str(1:len(dstring(1)))   ! Set PIPP description string

! Read in time-dependent metadata:
call rdGCM_LMDnc(phase=1)

! If ~16 variables of "maximum" relevant size take up more than ~400 MB of
!  storage, divide the reading/processing/writing work into smaller chunks
!  (be careful of potential 4-byte integer overflow):
rpw_n_iter = int(int(ni*nj1*nk, kind=i8)*int(n_times*16*f4_B, kind=i8)/  &
     400000000_i8, kind=i4)+1
rpw_snapstride = n_times/rpw_n_iter+1

! Allocate memory for GCM fields:

allocate(t_m(ni,nj1,nk,rpw_snapstride),  &   ! Arrays for "raw" GCM values
     u_m(ni,nj1,nk,rpw_snapstride), v_m(ni,nj1,nk,rpw_snapstride),  &
     H2Ovmr_m(ni,nj1,nk,rpw_snapstride), dustmr_m(ni,nj1,nk,rpw_snapstride),  &
     H2Oimr_m(ni,nj1,nk,rpw_snapstride), H2Odmr_m(ni,nj1,nk,rpw_snapstride),  &
     gt_SBS_m(ni,nj1,nk_SBS,rpw_snapstride), sp_m(ni,nj1,rpw_snapstride),  &
     ghi_sfc_m(ni,nj1,rpw_snapstride), dust_asfc_m(ni,nj1,rpw_snapstride),  &
     H2Oi_asfc_m(ni,nj1,rpw_snapstride), H2Od_asfc_m(ni,nj1,rpw_snapstride),  &
     gt_sfc_m(ni,nj1,rpw_snapstride), gi_sfc_m(ni,nj1,rpw_snapstride),  &
     p_m(ni,nj1,nk,rpw_snapstride), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'B')

allocate(u_a(ni,nj1,nk), v_a(ni,nj1,nk),  &     ! Arrays for "adjusted" values
     p_a(ni,nj1,nk), t_a(ni,nj1,nk), bpz_a(ni,nj1,nk), gt_sfc_a(ni,nj1),  &
     gt_SBS_a(ni,nj1,nk_SBS), sp_a(ni,nj1), rho_a(ni,nj1,nk),  &
     H2Ovmc_a(ni,nj1,nk), dustmc_a(ni,nj1,nk), H2Oimc_a(ni,nj1,nk),  &
     H2Odmc_a(ni,nj1,nk), ghi_sfc_a(ni,nj1), dust_asfc_a(ni,nj1),  &
     H2Oi_asfc_a(ni,nj1), H2Od_asfc_a(ni,nj1), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'C')

IO_reclen = determine_IO_reclen(PIPP_data(1:rec_size(1)))

i_sol = 1
rpw_snap_e = 0  ! Initial value
do i_iter=1,rpw_n_iter
   ! Snapshots to read/process/write in this chunk: 
   rpw_snap_b = rpw_snap_e+1
   rpw_snap_e = min(rpw_snap_b+rpw_snapstride-1, n_times)

   ! Read in relevant GCM data:
   write (*, '(/,a,/)') 'Reading GCM output ...'
   call rdGCM_LMDnc(phase=2)

   ! Process each timestep in sequence; write PIPP-format datafile: 

      ! Open PIPP-format datafile:
   open(IOunit, file=trim(PIPPdat), status='unknown', form='unformatted',  &
        access='direct', recl=IO_reclen)

   i_it = 0
   do i_snap=rpw_snap_b,rpw_snap_e
      i_it = i_it+1

      write (*, '(a,i5,a,f7.2,a)')  &          ! Report progress to user:
           'Processing and writing PIPP-format: SNAPSHOT = ', i_snap,  &
           ', LS =', lsubs_ts(i_snap,1), ' ...'

      if (i_snap > 1) then
         if (utc_ts(i_snap,1)-utc_ts(i_snap-1,1) < ZERO_f4) then
            i_sol = i_sol+1; sol_ts(i_snap,1) = i_sol
         end if
      end if

      u_a(:,:,:) = u_m(:,:,:,i_it); v_a(:,:,:) = v_m(:,:,:,i_it)

      t_a(:,:,:) = t_m(:,:,:,i_it); gt_SBS_a(:,:,:) = gt_SBS_m(:,:,:,i_it)

      do j=1,nj1
         do i=1,ni
            if (gi_sfc_m(i,j,i_it) > 1.e-9_f4) then
               gt_sfc_a(i,j) = -gi_sfc_m(i,j,i_it)
            else
               gt_sfc_a(i,j) = gt_sfc_m(i,j,i_it)
            end if
         end do
      end do

      ghi_sfc_a(:,:) = ghi_sfc_m(:,:,i_it)
      dust_asfc_a(:,:) = dust_asfc_m(:,:,i_it)
      H2Oi_asfc_a(:,:) = H2Oi_asfc_m(:,:,i_it)
      H2Od_asfc_a(:,:) = H2Od_asfc_m(:,:,i_it)

      p_a(:,:,:) = p_m(:,:,:,i_it)
      sp_a(:,:) = sp_m(:,:,i_it)
      call bpz_calc   ! Calculate bpz_a (body-potential height)

      rho_a(:,:,:) = p_a(:,:,:)/(R_air*t_a(:,:,:))

      ! Convert mass mixing ratios to mass concentrations:
      H2Ovmc_a(:,:,:) = H2Ovmr_m(:,:,:,i_it)*rho_a(:,:,:)
      dustmc_a(:,:,:) = dustmr_m(:,:,:,i_it)*rho_a(:,:,:)
      H2Oimc_a(:,:,:) = H2Oimr_m(:,:,:,i_it)*rho_a(:,:,:)
      H2Odmc_a(:,:,:) = H2Odmr_m(:,:,:,i_it)*rho_a(:,:,:)

      ! Fill PIPP 4D data cube with one timestep's data:
  
      call PIPPf_w_fill(t_a(:,:,:), 1)
      call PIPPf_w_fill(p_a(:,:,:), 2)
      call PIPPf_w_fill(u_a(:,:,:), 3)
      call PIPPf_w_fill(v_a(:,:,:), 4)
      call PIPPf_w_fill(bpz_a(:,:,:), 5)
      call PIPPf_w_fill(H2Ovmc_a(:,:,:), 6)
      call PIPPf_w_fill(dustmc_a(:,:,:), 7)
      call PIPPf_w_fill(H2Oimc_a(:,:,:), 8)
      call PIPPf_w_fill(H2Odmc_a(:,:,:), 9)
      call PIPPf_w_fill(gt_SBS_a(:,:,:), 10)
      call PIPPf_w_fill(gt_sfc_a(:,:), 11)
      call PIPPf_w_fill(ghi_sfc_a(:,:), 12)
      call PIPPf_w_fill(dust_asfc_a(:,:), 13)
      call PIPPf_w_fill(H2Oi_asfc_a(:,:), 14)
      call PIPPf_w_fill(H2Od_asfc_a(:,:), 15)

      ! Write the timestep's data to PIPP-format file:

      call PIPPf_w_data(IOunit, i_snap)

   end do
end do

call mNc_recycle_f_md_obj(file_in)  ! Close input datafile
close(IOunit)     ! Close output (PIPP-format) datafile

! Write PIPP-format header file:

do j=1,nj1
   do i=1,ni
      lat(i,j,1) = lat0(j)
      lon(i,j,1) = lon0(i)
   end do
end do

do iv=1,n_var
   vtype(iv,1) = vt(iv)
end do

call PIPPf_w_head(PIPPhdr)

! Open special header file; write SBS depth info:

open(IOunit,file=trim(PIPPhdr)//'.tzl', status='unknown', form='unformatted')
write (IOunit) nk_SBS
write (IOunit) zlevSBS
close(IOunit)

! Write GrADS descriptor file:

open(IOunit, file=gradsctlo(1:len_trim(gradsctlo)), status='unknown',  &
     form='formatted')

lastslash = index(PIPPdat, '/', .TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(PIPPdat)
dfname = PIPPdat(lastslash+1:fnwp_len)  ! no path
write (IOunit, '(a,2x,a)') 'dset', '^'//dfname(1:len_trim(dfname))

write (IOunit, '(a,2x,e13.6)') 'undef', undef_value_f4
write (IOunit, '(a)') 'title GCM data'
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'xdef', ni, 'linear',  &
     lon0(1), 360._f4/real(ni, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'ydef', nj1, 'linear',  &
     lat0(1), 180._f4/real(nj1-1, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,i1,2x,i1)') 'zdef', nk, 'linear', 1, 1
i = max(1, nint(snap_tsep_bmin))  ! [body-minutes]
if (i <= 9) then
   write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', n_times,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else if (i <= 99) then
   write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', n_times,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else
   i = nint(snap_tsep_bmin/60._f4)
   if (i <= 9) then
      write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', n_times,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   else
      write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', n_times,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   end if
end if
write (IOunit, '(a,2x,i2)') 'vars', n_var
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'temp', nk, 99, 'temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'p', nk, 99, 'atm pressure [Pa]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'u', nk, 99, 'u-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'v', nk, 99, 'v-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'bz', nk, 99, 'body-potential height [m]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Ov_mc', nk, 99,  &
     'H2O_vapor mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'dust_mc', nk, 99,  &
     'dust aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Oi_mc', nk, 99,  &
     'H2O_ice aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Od_mc', nk, 99,  &
     'H2Oi_dust aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_SBS', nk_SBS, 99,  &
     'subsfc temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_sfc', 0, 99,  &
     'sfc temp or ice mass [K or -kg]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'ghi_sfc', 0, 99,  &
     'sfc H2O ice mass [kg/m^2]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'dust_asfc', 0, 99,  &
     'sfc dust aer mass [kg/m^2]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Oi_asfc', 0, 99,  &
     'sfc H2O_ice aer mass [kg/m^2]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Od_asfc', 0, 99,  &
     'sfc H2Oi_dust aer mass [kg/m^2]'

write (IOunit, '(a,/)') 'endvars'
close(IOunit)

call PIPPf_dealloc()
deallocate(t_m, u_m, v_m, p_m, sp_m, gt_sfc_m, gi_sfc_m, gt_SBS_m, ghi_sfc_m,  &
     H2Ovmr_m, dustmr_m, H2Oimr_m, H2Odmr_m, dust_asfc_m, H2Oi_asfc_m,  &
     H2Od_asfc_m, u_a, v_a, p_a, sp_a, bpz_a, t_a, gt_sfc_a, gt_SBS_a, rho_a,  &
     ghi_sfc_a, H2Ovmc_a, dustmc_a, H2Oimc_a, H2Odmc_a, dust_asfc_a,  &
     H2Oi_asfc_a, H2Od_asfc_a, lat0, lon0, nzi, vt, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'A')

write (*, '(/,3(a,/))') 'Created the following file (sets): ',  &
     trim(gradsctlo), trim(outprfx)//'.*'

stop '** Normal completion **'


contains


!```````````````````````````````````````````````````````````````````
subroutine rdGCM_LMDnc(phase)

!+ Reads data from NASA ARC MGCM output files (v2.3+; NetCDF-format output)

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_phys_constants_mod, only: R_univ_gas_f4
use univ_convert_units_mod, only: g_to_kg_f4
use manage_NetCDF_classic_mod, only: mNc_var_dims, mNc_populate_f_md_obj_r,  &
     mNc_dim_val, mNc_var_idx
use manage_NetCDF_classic_rv_mod, only: mNc_read_var

implicit none

!=== Argument declarations:

integer, intent(IN) :: phase

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'rdGCM_LMDnc'

integer :: e, k, j
integer, dimension(3) :: start_t2Ds
integer, dimension(4) :: start_t3Ds
integer, dimension(5) :: start_t4Ds

integer, allocatable, dimension(:) :: tmp_dim
real(f4), pointer, dimension(:) :: p_tmp_1d => NULL()
real(f4), pointer, dimension(:,:) :: p_tmp_2d => NULL()
real(f4), pointer, dimension(:,:,:) :: p_tmp_3d => NULL()

!=== Executable statements:

select case (phase)
case (0)    ! Read time-independent "header" data

   call mNc_populate_f_md_obj_r(trim(dfname_wp), file_in)

   allocate(tmp_dim(1), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'Ar')

   tmp_dim = mNc_var_dims(file_in, 'latitude', 1)
   nj1 = tmp_dim(1)  ! Input model output includes both poles

   tmp_dim = mNc_var_dims(file_in, 'longitude', 1)
   ni = tmp_dim(1)-1   ! Input model output includes longitude wrapping

   tmp_dim = mNc_var_dims(file_in, 'altitude', 1)
   nk = tmp_dim(1)

   tmp_dim = mNc_var_dims(file_in, 'soildepth', 1)
   nk_SBS = tmp_dim(1)

   n_times = mNc_dim_val(file_in, 'Time')

   call mNc_read_var(file_in, 'controle', p_tmp_1d)
   g = p_tmp_1d(7)   ! [m/s^2]
   R_air = R_univ_gas_f4/  &  ! [J/(kg.K)]
        (p_tmp_1d(8)*g_to_kg_f4)    !  Molar mass of "air" [kg/mol]
   cp_air = R_air/p_tmp_1d(9)  ! [J/(kg.K)]

   allocate(zlevSBS(nk_SBS+1), lat0(nj1), lon0(ni), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'Br')

   call mNc_read_var(file_in, 'latitude', p_tmp_1d)
   do j=1,nj1     ! Reverse so that first 'j' index is southernmost
      lat0(1+nj1-j) = p_tmp_1d(j)
   end do

   call mNc_read_var(file_in, 'longitude', p_tmp_1d)
   lon0(:) = p_tmp_1d(1:ni)

   call mNc_read_var(file_in, 'soildepth', p_tmp_1d)
   do k=1,nk_SBS
      zlevSBS(1+nk_SBS-k) = -p_tmp_1d(k)  ! Reverse_order/negate (PRAMS use)
   end do
   zlevSBS(nk_SBS+1) = 0._f4

   deallocate(tmp_dim, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'Ar')

case (1)  ! Read time-dependent "header" data

   call mNc_read_var(file_in, 'Ls', p_tmp_1d)
   lsubs_ts(1:n_times,1) = p_tmp_1d(:)   ! Full, not a "step function"

   call mNc_read_var(file_in, 'phisinit', p_tmp_2d)   ! [m]
   do j=1,nj1   ! Reverse so that first 'j' index is southernmost
      topo(:,1+nj1-j,1) = p_tmp_2d(1:ni,j)/g   ! [m^2/s^2 -> m]
   end do

   call mNc_read_var(file_in, 'Time', p_tmp_1d)   ! [body-days UT]
   utc_ts(1:n_times,1) = mod(p_tmp_1d(:), 1._f4)*24._f4   ! [body-hours UT]
   snap_tsep_bmin = max(utc_ts(2,1)-utc_ts(1,1), utc_ts(2,1)-utc_ts(1,1))*  &
        60._f4  ! [body-minutes]

case (2)  ! Read desired GCM output fields ('rpw_snapstride' snapshots per call)

   i_it = 0
   do i_snap=rpw_snap_b,rpw_snap_e  ! Read each snapshot in sequence
      i_it = i_it+1

      ! Report progress to user:
      write (*, '(a,i5,a)') 'Reading SNAPSHOT = ', i_snap, ' ...'

      start_t2Ds = (/ 1, 1, i_snap /)
      start_t3Ds = (/ 1, 1, 1, i_snap /)
      start_t4Ds = (/ 1, 1, 1, 1, i_snap /)

    !*** Note that there is no need to reverse the vertical order of 3D fields.

      call mNc_read_var(file_in, 'temp', p_tmp_3d, opt_start=start_t3Ds) 
      do k=1,nk
         do j=1,nj1   ! Reverse so that first 'j' index is southernmost
            t_m(:,1+nj1-j,k,i_it) = p_tmp_3D(1:ni,j,k)      ! [K]
         end do
      end do
      
      call mNc_read_var(file_in, 'u', p_tmp_3d, opt_start=start_t3Ds) 
      do k=1,nk
         do j=1,nj1   ! Reverse so that first 'j' index is southernmost
            u_m(:,1+nj1-j,k,i_it) = p_tmp_3D(1:ni,j,k)      ! [m/s]
         end do
      end do

      call mNc_read_var(file_in, 'v', p_tmp_3d, opt_start=start_t3Ds) 
      do k=1,nk
         do j=1,nj1   ! Reverse so that first 'j' index is southernmost
            v_m(:,1+nj1-j,k,i_it) = p_tmp_3D(1:ni,j,k)      ! [m/s]
         end do
      end do

      call mNc_read_var(file_in, 'p', p_tmp_3d, opt_start=start_t3Ds)
      do k=1,nk
         do j=1,nj1   ! Reverse so that first 'j' index is southernmost
            p_m(:,1+nj1-j,k,i_it) = p_tmp_3D(1:ni,j,k)      ! [Pa]
         end do
      end do

      if (mNc_var_idx(file_in, 'h2o_vap', .TRUE.) /= -1) then  ! Var found
         call mNc_read_var(file_in, 'h2o_vap', p_tmp_3d, opt_start=start_t3Ds)
         do k=1,nk
            do j=1,nj1   ! Reverse so that first 'j' index is southernmost 
               H2Ovmr_m(:,1+nj1-j,k,i_it) = p_tmp_3d(1:ni,j,k)    ! [kg/kg_air]
            end do
         end do
      else
         H2Ovmr_m(:,:,:,i_it) = 0._f4    ! [kg/kg_air]
      end if

      if (mNc_var_idx(file_in, 'h2o_ice', .TRUE.) /= -1) then  ! Var found
         call mNc_read_var(file_in, 'h2o_ice', p_tmp_3d, opt_start=start_t3Ds)
         do k=1,nk
            do j=1,nj1   ! Reverse so that first 'j' index is southernmost 
               H2Oimr_m(:,1+nj1-j,k,i_it) = p_tmp_3d(1:ni,j,k)   ! [kg/kg_air]
            end do
         end do
      else
         H2Oimr_m(:,:,:,i_it) = 0._f4
      end if

!% No dust or H2O-ice-cloud dust mass in output!!!!
      dustmr_m(:,:,:,i_it) = 0._f4   ! [kg/kg_air]
      H2Odmr_m(:,:,:,i_it) = 0._f4   ! [kg/kg_air]

      call mNc_read_var(file_in, 'ps', p_tmp_2d, opt_start=start_t2Ds)
      do j=1,nj1   ! Reverse so that first 'j' index is southernmost 
         sp_m(:,1+nj1-j,i_it) = p_tmp_2D(1:ni,j)      ! [K]
      end do

      call mNc_read_var(file_in, 'tsurf', p_tmp_2d, opt_start=start_t2Ds) 
      do j=1,nj1   ! Reverse so that first 'j' index is southernmost
         gt_sfc_m(:,1+nj1-j,i_it) = p_tmp_2D(1:ni,j)      ! [K]
      end do

      call mNc_read_var(file_in, 'co2_ice_surf', p_tmp_2d, opt_start=start_t2Ds)
      do j=1,nj1   ! Reverse so that first 'j' index is southernmost
         gi_sfc_m(:,1+nj1-j,i_it) = p_tmp_2D(1:ni,j)      ! [kg/m^2]
      end do

      if (mNc_var_idx(file_in, 'h2o_ice_surf', .TRUE.) /= -1) then
         call mNc_read_var(file_in, 'h2o_ice_surf', p_tmp_2d,  &
              opt_start=start_t2Ds)
         do j=1,nj1   ! Reverse so that first 'j' index is southernmost
            ghi_sfc_m(:,1+nj1-j,i_it) = p_tmp_2d(1:ni,j)  ! [kg/m^2]
         end do
      else
         ghi_sfc_m(:,:,i_it) = 0._f4  ! [kg/m^2]
      end if

!% No surface cloud particle mass in output!!!!
      dust_asfc_m(:,:,i_it) = 0._f4   ! [kg/m^2]
      H2Oi_asfc_m(:,:,i_it) = 0._f4   ! [kg/m^2]
      H2Od_asfc_m(:,:,i_it) = 0._f4   ! [kg/m^2]

      call mNc_read_var(file_in, 'tsoil', p_tmp_3d, opt_start=start_t3Ds) 
      do k=1,nk_SBS
         do j=1,nj1   ! Reverse so that first 'j' index is southernmost
            gt_SBS_m(:,1+nj1-j,1+nk_SBS-k,i_it) = p_tmp_3D(1:ni,j,k)  ! [K]
         end do
      end do

   end do

end select

end subroutine rdGCM_LMDnc


!```````````````````````````````````````````````````````````````````
subroutine bpz_calc

!+ Calculates vertical pressure levels and body-potential heights

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: i, j, k
real(f4) :: H, tbot, pbot, bzsum, tavg

!=== Executable statements:
     
do i=1,ni
   do j=1,nj1

      ! Compute body-potential near surface from surface pressure

      bzsum = 0._f4
      pbot = sp_a(i,j)  ! Begin integration at surface pressure
      tbot = t_a(i,j,1)   ! Not at all the best value for this, but oh well
      do k=1,nk   ! Loop over all GCM layers 
         tavg = (tbot+t_a(i,j,k))*0.5_f4 ! Average tempk for the current layer
         H = (-R_air/g)*tavg*log(p_a(i,j,k)/pbot)  ! Delta-body-potential height
         bzsum = bzsum+H       ! Body-potential height of current layer *AGL*
         bpz_a(i,j,k) = bzsum  !   (Ignore whatever bpz GCM has)
         pbot = p_a(i,j,k)   ! These will become the lower boundary for the
         tbot = t_a(i,j,k)   !   next iteration
      end do

   end do
end do

end subroutine bpz_calc


!```````````````````````````````````````````````````````````````````
subroutine usage

!+ Prints program usage information

use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

write (*, '(/,a,/)')  &
     'USAGE:  Must be started from run_prep, or a similar helper script'
call exception('usage', FATAL)

end subroutine usage


end program split_LMD_ncoutp
