#!/usr/bin/env python
"""
Sets up and executes a PRAMS preprocessor run.

This program requires either Python version 2.6+ or 3.2+, and is importable as a
python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import sys
import os
import subprocess
import getopt
import re
import io
import shutil

#------------------------------------------------------------------------------
def rm_f(fpath):
    """
    Emulates 'rm -f' functionality.
    """

    if os.path.isfile(fpath) or os.path.islink(fpath):
        os.remove(fpath)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def main(input_fpfx, output_path, use_debugger, prep_IDstr, exe_file_pfx,
         nc_subset):
    """Driver routine."""

    #=== Define some useful parameters:

    exe_file_pfx_full = os.path.abspath(exe_file_pfx)

    generated_nml_fn_pfx = "auto_gen-prep.nml"

    generated_nml_fpath = os.path.abspath(generated_nml_fn_pfx)
    input_fpath = os.path.abspath(input_fpfx)
    if "NOT_GIVEN__" in output_path:
        output_path = os.path.dirname(input_fpath)
    else:
        output_path = os.path.abspath(output_path)

    subset_cmd_preamble = "ncea -F -d "

    # Attempt to determine what type of input files were specified, and whether
    #  all needed files/information are present/readable; if successful, process
    #  the data and output it in a PRAMS-readable format:

    proc_fp_pfx = os.path.join(output_path,
                               "proc_"+os.path.basename(input_fpath))

    input_header_fpath = input_fpath+".hdr"     # Header file name to check for
    if os.path.isfile(input_header_fpath):
        print("Input header file found; assuming data_file+header_file format.")
        input_data_fpath = input_fpath+".dat"      # Create data file name
        if not os.path.isfile(input_data_fpath):
            print("ERROR: data file ("+input_data_fpath+ \
                  ") was not found or is unreadable.")
            sys.exit(1)

        perform_subset = False   # Dataset subsetting not supported
        if "NOT_GIVEN__" not in nc_subset:
            print("ERROR: This dataset type does not support subsetting -- "
                  "please remove the '--nc-subset' option and try again.")
            sys.exit(1)

        # NASA Ames MGCM (v1.7.3; TIM's "new" output):
        print("Using NASA Ames MGCM (v1.7.3; \"new\" output) processor ...")
        dstring = "01-A_MGCM_1.7.3_newoutp"
        exe_sfx = "_A173newoutp"
        aux_cmd_list = []   # None needed for this application

        text_to_write = \
            "&prep_args\n" \
            "   hfname_wp = \'"+input_header_fpath+"\',\n" \
            "   dfname_wp = \'"+input_data_fpath+"\',\n" \
            "   outprfx = \'"+proc_fp_pfx+"\',\n" \
            "   desc_str = \'"+dstring+"\' /\n"

    elif os.path.isfile(input_fpath+".nc"):
        print("Assuming an \"all-inclusive\" NetCDF-format datafile.")
        input_data_fpath = input_fpath+".nc"

        aux_cmd_list = []

        # Determine some selected info related to operating on this
        #  NetCDF-format file:
        nc_md, snap_beg_Fi, snap_end_Fi, perform_subset = info_for_nc_use(
            input_data_fpath, nc_subset)

        if perform_subset:
            proc_sfx = ".snaps_"+str(snap_beg_Fi)+"to"+str(snap_end_Fi)
            tmp_fpathA = os.path.join(output_path,
                                      os.path.basename(input_fpath))+ \
                                      proc_sfx+".nc"
            rm_f(tmp_fpathA)  # Remove old debris, necessary for 'nco' commands
            subset_cmd_postamble = ","+str(snap_beg_Fi)+","+str(snap_end_Fi)+ \
                                   " "+input_data_fpath+" "+tmp_fpathA

        if "controle(lentable)" in nc_md:

            #== UK MGCM NetCDF processor:
            print("Using UK MGCM NetCDF processor ...")
            dstring = "78-UK_MGCM_ncoutp"
            exe_sfx = "_UK_ncoutp"

            perform_subset = False   # Dataset subsetting not supported
            if "NOT_GIVEN__" not in nc_subset:
                print("ERROR: This dataset type does not support subsetting -- "
                      "please remove the '--nc-subset' option and try again.")
                sys.exit(1)

            text_to_write = \
                            "&prep_args\n" \
                            "   dfname_wp = \'"+input_data_fpath+"\',\n" \
                            "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                            "   desc_str = \'"+dstring+"\' /\n"

        elif "lyr_mid:name_in_model_code" in nc_md:
            #== NASA Ames MGCM 2.3(+) NetCDF processor:
            print("Using NASA Ames MGCM v2.3(+) NetCDF processor ...")
            dstring = "76-A_MGCM_2.3nc"
            exe_sfx = "_A23nc"
            if perform_subset:
                tmp_str = subset_cmd_preamble+"simtime"+subset_cmd_postamble
                aux_cmd_list.append(tmp_str.split())
                input_data_fpath = tmp_fpathA   # Read from the subset file
                proc_fp_pfx += proc_sfx

            text_to_write = \
                            "&prep_args\n" \
                            "   dfname_wp = \'"+input_data_fpath+"\',\n" \
                            "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                            "   desc_str = \'"+dstring+"\' /\n"

        elif "lyr_mid:name_in_model" in nc_md:
            #== NASA Ames MGCM 2.1 v23(+) NetCDF processor:
            print("Using NASA Ames MGCM v2.1 v23(+) NetCDF processor ...")
            dstring = "79-A_MGCM_2.1_v23nc"
            exe_sfx = "_A21v23nc"
            if perform_subset:
                tmp_str = subset_cmd_preamble+"time"+subset_cmd_postamble
                aux_cmd_list.append(tmp_str.split())
                input_data_fpath = tmp_fpathA   # Read from the subset file
                proc_fp_pfx += proc_sfx

            text_to_write = \
                            "&prep_args\n" \
                            "   dfname_wp = \'"+input_data_fpath+"\',\n" \
                            "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                            "   desc_str = \'"+dstring+"\' /\n"

        elif "aire(" in nc_md:
            #== LMD (c.2014) NetCDF processor (partially preprocessed input):
            print("Using LMD MGCM NetCDF processor (partially preprocessed input) ...")
            dstring = "75-LMD_MGCM_ncoutp"
            exe_sfx = "_LMD_ncoutp"
            aux_cmd_list = []

            if perform_subset:
                tmp_str = subset_cmd_preamble+"Time"+subset_cmd_postamble
                aux_cmd_list.append(tmp_str.split())
                input_data_fpath = tmp_fpathA   # Read from the subset file
                proc_fp_pfx += proc_sfx

            text_to_write = \
                            "&prep_args\n" \
                            "   dfname_wp = \'"+input_data_fpath+"\',\n" \
                            "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                            "   desc_str = \'"+dstring+"\' /\n"

        else:
            #== NASA Ames MGCM 2.1 NetCDF processor:
            print("Using NASA Ames MGCM v2.1 NetCDF processor ...")
            dstring = "77-A_MGCM_2.1_ncoutp"
            exe_sfx = "_A21ncoutp"
            if perform_subset:
                tmp_str = subset_cmd_preamble+"time"+subset_cmd_postamble
                aux_cmd_list.append(tmp_str.split())
                input_data_fpath = tmp_fpathA   # Read from the subset file
                proc_fp_pfx += proc_sfx

            text_to_write = \
                            "&prep_args\n" \
                            "   dfname_wp = \'"+input_data_fpath+"\',\n" \
                            "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                            "   desc_str = \'"+dstring+"\' /\n"
    
    elif os.path.isfile((input_fpath.replace("%", "fi"))+".nc"):
        print("Assuming \"multiple-file\" NetCDF-format dataset (LMD).")
        input_data_fpath = [input_fpath.replace("%", "fi")+".nc",
                            input_fpath.replace("%", "soil")+".nc"]
        if not os.path.isfile(input_data_fpath[1]):
            print("ERROR: data file ("+input_data_fpath[1]+ \
                  ") was not found or is unreadable.")
            sys.exit(1)

        aux_cmd_list = []

        # Determine some selected info related to operating on this
        #  NetCDF-format file:
        nc_md, snap_beg_Fi, snap_end_Fi, perform_subset = info_for_nc_use(
            input_data_fpath[0], nc_subset)

        #== LMD (c.2014) NetCDF processor (raw input):
        print("Using LMD MGCM NetCDF processor (raw input) ...")
        dstring = "75-LMD_MGCM_ncoutp"
        exe_sfx = "_LMD_ncoutp"
        aux_cmd_list = []

        if perform_subset:
            proc_sfx = ".snaps_"+str(snap_beg_Fi)+"to"+str(snap_end_Fi)
            tmp_fpathA = os.path.join(output_path,
                            os.path.basename(input_fpath.replace("%", "fi")))+ \
                                      proc_sfx+".nc"
            subset_cmd_postamble = ","+str(snap_beg_Fi)+","+str(snap_end_Fi)+" "
        else:
            tmp_fpathA = os.path.join(output_path,
                            os.path.basename(input_fpath.replace("%", "fi")))+ \
                                      ".snaps_1toEND.nc"
        tmp_fpathB = os.path.join(output_path, "tmp47629136_diagsoil.subset.nc")
        rm_f(tmp_fpathA)  # Remove old debris, necessary for 'nco' commands
        rm_f(tmp_fpathB)  # Remove old debris, necessary for 'nco' commands

        if perform_subset:
            # Command-line args for subsetting datafiles:
            tmp_str = subset_cmd_preamble+"Time"+subset_cmd_postamble+ \
                      input_data_fpath[0]+" "+tmp_fpathA
            aux_cmd_list.append(tmp_str.split())
            tmp_str = subset_cmd_preamble+"time"+subset_cmd_postamble+ \
                      input_data_fpath[1]+" "+tmp_fpathB
            aux_cmd_list.append(tmp_str.split())
        else:
            # Create datafile copies for editing/merging:
            shutil.copy(input_data_fpath[0], tmp_fpathA)
            shutil.copy(input_data_fpath[0], tmp_fpathB)

        # Minor metadata edits to allow merging of the files' data:
        cmd_preamble = "ncrename -O "
        tmp_str = cmd_preamble+"-v temp,tsoil "+tmp_fpathB
        aux_cmd_list.append(tmp_str.split())
        tmp_str = cmd_preamble+"-v tsurf,tsurf_copy "+tmp_fpathB
        aux_cmd_list.append(tmp_str.split())
        tmp_str = cmd_preamble+"-v time,Time "+tmp_fpathB
        aux_cmd_list.append(tmp_str.split())
        tmp_str = cmd_preamble+"-d time,Time "+tmp_fpathB
        aux_cmd_list.append(tmp_str.split())

        # Merge the files' data; remove unneeded temporary file:
        tmp_str = "ncks -A "+tmp_fpathB+" "+tmp_fpathA
        aux_cmd_list.append(tmp_str.split())
        rm_f(tmp_fpathB)

        # Modify to remove sentinel character(s):
        proc_fp_pfx = proc_fp_pfx.replace("%", "fi")
        if perform_subset:
            proc_fp_pfx += proc_sfx

        text_to_write = \
                        "&prep_args\n" \
                        "   dfname_wp = \'"+tmp_fpathA+"\',\n" \
                        "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                        "   desc_str = \'"+dstring+"\' /\n"

    elif re.match(".*[0-9_][0-9]$", input_fpath):   # Ends with "##" or "_#" 
        input_data_fpath = input_fpath      # Create data file name
        if not os.path.isfile(input_data_fpath):
            print("ERROR: data file ("+input_data_fpath+ \
                  ") was not found or is unreadable.")
            sys.exit(1)
        print("Corresponding file with numeric suffix found; assuming "
              "'raw' output format.")

        perform_subset = False   # Dataset subsetting not supported
        if "NOT_GIVEN__" not in nc_subset:
            print("ERROR: This dataset type does not support subsetting -- "
                  "please remove the '--nc-subset' option and try again.")
            sys.exit(1)

        #== NASA Ames MGCM v2.1 "special" (made for SCCR by Jim S.) processor:
        print("Using NASA Ames MGCM v2.1 \"special\" processor ...")
        dstring = "02-A_MGCM_2.1_specoutp"
        exe_sfx = "_A21specoutp"
        aux_cmd_list = []   # None needed for this application

        text_to_write = \
                        "&prep_args\n" \
                        "   dfname_wp = \'"+input_data_fpath+"\',\n" \
                        "   outprfx = \'"+proc_fp_pfx+"\',\n" \
                        "   desc_str = \'"+dstring+"\' /\n"

    else:
        print("ERROR: no relevant file with the given prefix ("+ \
              input_fpath+") was found/readable.")
        sys.exit(1)

    # Write preprocessor namelist, if needed:
    if exe_sfx:
        with io.open(generated_nml_fpath, "wt", encoding="ascii") as text_f:
            text_f.write(text_to_write)

    # Perform any other command-line actions before running the preprocessor:
    if perform_subset:
        print("INFO: Subsetting data file(s) ...")
        for cmd_l in aux_cmd_list:
            subprocess.check_call(cmd_l)

    # Run the preprocessor itself, if needed:
    print("INFO: Invoking preprocessor executable ...")
    cmd_list = []
    if use_debugger:
        cmd_list.append("gdb")
    if exe_sfx:
        cmd_list.append(exe_file_pfx_full+exe_sfx)
        subprocess.check_call(cmd_list)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def info_for_nc_use(input_fpath, nc_subset):
    """Determine selected info related to operating on a NetCDF-format file."""

    # Returns a tuple containing 4 items (the first 3 may be "None"):
    #  1) A metadata string describing the NetCDF-format file's content
    #  2) The Fortran-style time index to begin subsetting at, if any
    #  3) The Fortran-style time index to end subsetting at, if any
    #  4) (boolean) Whether or not time-subsetting should be performed

    # Obtain information about the contents of the input file:
    p1 = subprocess.Popen(["ncdump", "-h", input_fpath],
                          stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                          universal_newlines=True)
    output, err = p1.communicate()

    # Extract any dimension information that is present:
    pattern_d = re.compile("dimensions:([\s\S]*)variables:")
    if pattern_d:
        dim_block_str = (pattern_d.search(output)).group(1)

    # Attempt to find the time dimension value:
    pattern_n = re.compile("(\d+)")
    ie = None
    if dim_block_str:
        for line in dim_block_str.splitlines():
            if "ime =" in line:
                ans = pattern_n.search(line).group(1)
                if ans:
                    dim_ie = int(ans)

    do_subset = False  # Assume that subsetting should not be performed
    ib_Fi = None
    ie_Fi = None

    if dim_ie:
        if "NOT_GIVEN__" not in nc_subset:
            # Parse nc_subset:
            be_str = nc_subset.split(',')
            ib_Fi = int(be_str[0])
            if "END" in be_str[1]:
                ie_Fi = dim_ie
            else:
                ie_Fi = int(be_str[1])

            # Sanity checks:
            if ib_Fi <= 0 or ie_Fi <= 0:
                print("ERROR: all indices of the '--nc-subset' range must be "
                      ">= 1")
                sys.exit(1)
            if ie_Fi > dim_ie:
                print("ERROR: the final index of the '--nc-subset' range must "
                      "be <= the dataset's time dimension value ("+ \
                      str(dim_ie)+").")
                sys.exit(1)
            if ie_Fi < ib_Fi:
                print("ERROR: the smaller index in the '--nc-subset' range "
                      "must be specified first.")
                sys.exit(1)

            # Should subsetting be performed?
            if ie_Fi == dim_ie:
                if ib_Fi > 1:   # Still a useful subset
                    do_subset = True
                else:
                    print("WARNING: '--nc-subset' range specifies the full "
                          "range in the dataset, so no subsetting will be "
                          "performed.")
            else:
                do_subset = True

    return (output, ib_Fi, ie_Fi, do_subset)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    """Prints usage information."""

    usage_msg = """
usage: run_prep.py [-h] [-d] [--output-dir=DIR] [--preptype=STRING]
                   [--exe-file-prefix=PFX] [--nc-subset=RANGE] INPUT_FPFX

Sets up and executes a PRAMS preprocessor run.

mandatory argument:
  INPUT_FPFX   The "shortest common denominator" of the names of the datafile(s)
                to be preprocessed.  Examples:
                  1) If the actual datafiles are:
                        /home/me/stuff/green.hdr, /home/me/stuff/green.dat
                      then INPUT_FPFX should be "/home/me/stuff/green"
                  2) If the actual datafile is:
                        /home/me/stuff/blue.nc
                      then INPUT_FPFX should be "/home/me/stuff/blue"
                  3) If the actual datafiles are:
                        /home/me/stuff/diagfi22.nc, /home/me/stuff/diagsoil22.nc
                      then INPUT_FPFX should be "/home/me/stuff/diag%22"
                  4) If the actual datafile is:
                        /home/me/stuff/run1_fort.11_26
                      then INPUT_FPFX should be "/home/me/stuff/run1_fort.11_26"

optional arguments:
  -h, --help   Show this help message and exit.
  -d, --debug   Run preprocessor inside a debugger utility (gdb)
                 (default: False)
  --output-dir=DIR   Path/directory in which to put processed (output) files
                      (default: the directory that the input file(s) are in)
  --preptype=STRING   **Not currently used**; an ID string that
                       specifies the type of preprocessor to invoke
                       (valid choices: )
  --exe-file-prefix=PFX   PRAMS preprocessor executable location/name prefix
                           (default: './exe/PIPP_p-split')

 ** For NetCDF-format input only, and requires the third-party 'nco' package
 **  (specifically, the nco programs 'ncea', 'ncrename', and 'ncks'):
  --nc-subset=RANGE   A snapshot (time index) range to extract from the input
                         datafile(s) before preprocessing
                         (default: '1,END'; examples: '3,44', '5,5', '3,END')
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd",
                                   ["help", "debug", "output-dir=",
                                    "preptype=", "exe-file-prefix=",
                                    "nc-subset="])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    output_path = "NOT_GIVEN__"
    use_debugger = False
    prep_IDstr = "NOT_GIVEN__"
    exe_file_pfx = os.path.join("exe", "PIPP_p-split")
    nc_subset = "NOT_GIVEN__"
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        elif o in ("-d", "--debug"):
            use_debugger = True
        elif o in ("--output-dir"):
            output_path = a
        elif o in ("--preptype"):
            prep_IDstr = a
        elif o in ("--exe-file-prefix"):
            exe_file_pfx = a
        elif o in ("--nc-subset"):
            nc_subset = a
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 1 or len(args) < 1):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)
    input_fpfx = args[0]

    # Run driver:
    main(input_fpfx, output_path, use_debugger, prep_IDstr, exe_file_pfx,
         nc_subset)
