!```````````````````````````````````````````````````````````````````
program topo_Mg128_convert_to_nc

!+ Converts MOLA 1/128 degree gridded PDS dataset to NetCDF format
!   NOTE: Creates a ~2 GiB file

use univ_kind_defs_mod, only: i2, f4
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use univ_math_constants_mod, only: ONE_f4
use univ_string_mod, only: char_array_to_string
use manage_NetCDF_classic_mod, only: mNc_MAX_NAME_LEN, mNc_file_md_Att_obj,  &
     mNc_file_metadata_obj, mNc_sync_with_file, mNc_populate_f_md_obj_w,  &
     mNc_write_var, mNc_recycle_f_md_obj, mNc_recycle_f_md_Att_obj,  &
     mNc_read_var, mNc_populate_f_md_obj_r, mNc_f_md_Att_populated
use read_Xendian_data_gf_mod, only: read_be_data_stream

implicit none

!== Local declarations:

integer, parameter :: header_n_B = 0,  &
                      n_files_to_process = 1,  &
                      n_d1_pts_tile = 11520, n_d2_pts_tile = 5632,  &
                      n_input_tiles = 4*4,  &
                      n_d1_pts = 360*128, n_d2_pts = 180*128
real(f4), parameter :: dlat = ONE_f4/128._f4, dlon = ONE_f4/128._f4

! Items for storage/processing of NetCDF metadata:
integer :: n_dims, n_vars, n_g_Atts, max_var_ndims, max_var_nAtts
character(len=mNc_MAX_NAME_LEN), allocatable, dimension(:) :: dim_name,  &
     var_name
integer, allocatable, dimension(:) :: dimv, var_type, var_ndims, var_nAtts
integer, allocatable, dimension(:,:) :: var_dimids
type(mNc_file_md_Att_obj), allocatable, dimension(:) :: g_Att
type(mNc_file_md_Att_obj), allocatable, dimension(:,:) :: var_Att
character(len=mNc_MAX_NAME_LEN) :: tmp_mNc_str

! Other items:

type(mNc_file_metadata_obj), save :: output_file_obj, input_file_obj

character(len=1), dimension(header_n_B) :: header_text_tmp
character(len=header_n_B) :: header_text

character(len=MAX_FILE_PATH_LEN), dimension(n_files_to_process) ::  &
     output_data_fn, desc_string
character(len=MAX_FILE_PATH_LEN), dimension(n_input_tiles) :: input_data_fn

integer, dimension(2,n_input_tiles) :: new_d2_idx, new_d1_idx
real(f4), dimension(2) :: lat_bounds, lon_bounds

integer :: i_d1, i_d2, cnt, i_file, i_tile
real(f4) :: lon_native_to_IAU2000

integer(i2), dimension(n_d1_pts_tile,n_d2_pts_tile) :: raw_data, proc_data
integer(i2), pointer, dimension(:,:) :: strip_64 => NULL()
integer(i2), dimension(128*360, 128*2) :: strip_128

!=== Executable statements:

!--- Dataset reference:
! Smith, D.E., M.T. Zuber, G.A. Neumann, E.A. Guinness, and S. Slavney, 
! Mars Global Surveyor Laser Altimeter Mission Experiment Gridded Data Record,
! MGS-M-MOLA-5-MEGDR-L3-V1.0, NASA Planetary Data System, 2003.

!--- Uses data obtained from 
!      http://geo.pds.nasa.gov/missions/mgs/megdr.html

input_data_fn(1) = 'megt44s000hb.img'
new_d2_idx(:,1) = (/ 256+1, 256+n_d2_pts_tile*1 /)
new_d1_idx(:,1) = (/ n_d1_pts_tile*2+1, n_d1_pts_tile*3 /)
input_data_fn(2) = 'megt00n000hb.img'
new_d2_idx(:,2) = (/ 256+n_d2_pts_tile*1+1, 256+n_d2_pts_tile*2 /)
new_d1_idx(:,2) = (/ n_d1_pts_tile*2+1, n_d1_pts_tile*3 /)
input_data_fn(3) = 'megt44n000hb.img'
new_d2_idx(:,3) = (/ 256+n_d2_pts_tile*2+1, 256+n_d2_pts_tile*3 /)
new_d1_idx(:,3) = (/ n_d1_pts_tile*2+1, n_d1_pts_tile*3 /)
input_data_fn(4) = 'megt88n000hb.img'
new_d2_idx(:,4) = (/ 256+n_d2_pts_tile*3+1, 256+n_d2_pts_tile*4 /)
new_d1_idx(:,4) = (/ n_d1_pts_tile*2+1, n_d1_pts_tile*3 /)
input_data_fn(5) = 'megt44s090hb.img'
new_d2_idx(:,5) = (/ 256+1, 256+n_d2_pts_tile*1 /)
new_d1_idx(:,5) = (/ n_d1_pts_tile*3+1, n_d1_pts_tile*4 /)
input_data_fn(6) = 'megt00n090hb.img'
new_d2_idx(:,6) = (/ 256+n_d2_pts_tile*1+1, 256+n_d2_pts_tile*2 /)
new_d1_idx(:,6) = (/ n_d1_pts_tile*3+1, n_d1_pts_tile*4 /)
input_data_fn(7) = 'megt44n090hb.img'
new_d2_idx(:,7) = (/ 256+n_d2_pts_tile*2+1, 256+n_d2_pts_tile*3 /)
new_d1_idx(:,7) = (/ n_d1_pts_tile*3+1, n_d1_pts_tile*4 /)
input_data_fn(8) = 'megt88n090hb.img'
new_d2_idx(:,8) = (/ 256+n_d2_pts_tile*3+1, 256+n_d2_pts_tile*4 /)
new_d1_idx(:,8) = (/ n_d1_pts_tile*3+1, n_d1_pts_tile*4 /)
input_data_fn(9) = 'megt44s180hb.img'
new_d2_idx(:,9) = (/ 256+1, 256+n_d2_pts_tile*1 /)
new_d1_idx(:,9) = (/ 1, n_d1_pts_tile*1 /)
input_data_fn(10) = 'megt00n180hb.img'
new_d2_idx(:,10) = (/ 256+n_d2_pts_tile*1+1, 256+n_d2_pts_tile*2 /)
new_d1_idx(:,10) = (/ 1, n_d1_pts_tile*1 /)
input_data_fn(11) = 'megt44n180hb.img'
new_d2_idx(:,11) = (/ 256+n_d2_pts_tile*2+1, 256+n_d2_pts_tile*3 /)
new_d1_idx(:,11) = (/ 1, n_d1_pts_tile*1 /)
input_data_fn(12) = 'megt88n180hb.img'
new_d2_idx(:,12) = (/ 256+n_d2_pts_tile*3+1, 256+n_d2_pts_tile*4 /)
new_d1_idx(:,12) = (/ 1, n_d1_pts_tile*1 /)
input_data_fn(13) = 'megt44s270hb.img'
new_d2_idx(:,13) = (/ 256+1, 256+n_d2_pts_tile*1 /)
new_d1_idx(:,13) = (/ n_d1_pts_tile*1+1, n_d1_pts_tile*2 /)
input_data_fn(14) = 'megt00n270hb.img'
new_d2_idx(:,14) = (/ 256+n_d2_pts_tile*1+1, 256+n_d2_pts_tile*2 /)
new_d1_idx(:,14) = (/ n_d1_pts_tile*1+1, n_d1_pts_tile*2 /)
input_data_fn(15) = 'megt44n270hb.img'
new_d2_idx(:,15) = (/ 256+n_d2_pts_tile*2+1, 256+n_d2_pts_tile*3 /)
new_d1_idx(:,15) = (/ n_d1_pts_tile*1+1, n_d1_pts_tile*2 /)
input_data_fn(16) = 'megt88n270hb.img'
new_d2_idx(:,16) = (/ 256+n_d2_pts_tile*3+1, 256+n_d2_pts_tile*4 /)
new_d1_idx(:,16) = (/ n_d1_pts_tile*1+1, n_d1_pts_tile*2 /)

output_data_fn(1) = 'topo-global-MOLA_128.nc'
desc_string(1) = 'topographic height above areoid (MGS MOLA-based)'

do i_file=1,n_files_to_process

   ! Some metadata to include in the new NetCDF file:

   lon_native_to_IAU2000 = 0._f4   ! Already IAU2000 coordinates

   lat_bounds(:) = (/ -90.+dlat/2., 90.-dlat/2. /)     ! [degrees_N]
   lon_bounds(:) = (/ -180.+dlon/2., 180.-dlon/2. /)   ! [degrees_E]

   ! Define and write a new NetCDF file:

   call define_output_file((i_file == 1), desc_string(i_file))

   call mNc_populate_f_md_obj_w(output_data_fn(i_file), output_file_obj,  &
        n_dims, n_vars, n_g_Atts, dim_name, dimv, g_Att, var_name, var_type,  &
        var_ndims, var_nAtts, var_dimids, var_Att)

   do i_tile=1,n_input_tiles

print *, 'Reading input file', i_tile, trim(input_data_fn(i_tile))
      ! Read in the raw dataset (skipping the header):
      call read_be_data_stream(raw_data, trim(input_data_fn(i_tile)),  &
           header_n_B)

      ! Need to reorder dataset points in latitude:
      
      do i_d1=1,n_d1_pts_tile   ! *DO NOT* need to reorder points in longitude

         cnt = 0
         do i_d2=n_d2_pts_tile,1,-1
            cnt = cnt+1
            proc_data(i_d1,cnt) = raw_data(i_d1,i_d2)
         end do

      end do

print *, 'Writing tile', i_tile, new_d1_idx(1,i_tile), new_d2_idx(1,i_tile)
      call mNc_write_var(output_file_obj, 'topo', proc_data,  &
           opt_start=(/ new_d1_idx(1,i_tile), new_d2_idx(1,i_tile) /),  &
           opt_count=(/ n_d1_pts_tile, n_d2_pts_tile /))

   end do

   ! Use 1/64 degree data to fill areas poleward of 88 deg latitude:

   call mNc_populate_f_md_obj_r('topo-global-MOLA_64.nc', input_file_obj)

      ! South polar "strip":
   call mNc_read_var(input_file_obj, 'topo', strip_64,  &
        opt_start=(/ 1, 1 /), opt_count=(/ 64*360, 64*2 /))
   do i_d2=1,size(strip_64, 2)
      do i_d1=1,size(strip_64, 1)
         strip_128(2*i_d1-1,2*i_d2) = strip_64(i_d1,i_d2)
         strip_128(2*i_d1-1,2*i_d2-1) = strip_64(i_d1,i_d2)
         strip_128(2*i_d1,2*i_d2-1) = strip_64(i_d1,i_d2)
         strip_128(2*i_d1,2*i_d2) = strip_64(i_d1,i_d2)
      end do
   end do
   print *, 'Writing strip S'
   call mNc_write_var(output_file_obj, 'topo', strip_128,  &
        opt_start=(/ 1, 1 /),  &
        opt_count=(/ 128*360, 128*2 /))

      ! North polar "strip":
   call mNc_read_var(input_file_obj, 'topo', strip_64,  &
        opt_start=(/ 1, 178*64+1 /), opt_count=(/ 64*360, 64*2 /))
   do i_d2=1,size(strip_64, 2)
      do i_d1=1,size(strip_64, 1)
         strip_128(2*i_d1-1,2*i_d2) = strip_64(i_d1,i_d2)
         strip_128(2*i_d1-1,2*i_d2-1) = strip_64(i_d1,i_d2)
         strip_128(2*i_d1,2*i_d2-1) = strip_64(i_d1,i_d2)
         strip_128(2*i_d1,2*i_d2) = strip_64(i_d1,i_d2)
      end do
   end do
   print *, 'Writing strip N'
   call mNc_write_var(output_file_obj, 'topo', strip_128,  &
        opt_start=(/ 1, 178*128+1 /),  &
        opt_count=(/ 128*360, 128*2 /))

   call mNc_recycle_f_md_obj(input_file_obj)   ! Close file, free object memory

   ! Tidy up ...

   call mNc_sync_with_file(output_file_obj)   ! Optional

   ! Finalize file, free object memory:
   call mNc_recycle_f_md_obj(output_file_obj) 

   ! Must "recycle" the attribute objects before reusing:
   do i_d2=1,size(var_Att,2)
      do i_d1=1,size(var_Att,1)
         if (mNc_f_md_Att_populated(var_Att(i_d1,i_d2)))  &
              call mNc_recycle_f_md_Att_obj(var_Att(i_d1,i_d2))
      end do
   end do
   do i_d1=1,size(g_Att,1)
      if (mNc_f_md_Att_populated(g_Att(i_d1)))  &
           call mNc_recycle_f_md_Att_obj(g_Att(i_d1))
   end do

end do

stop

contains


!```````````````````````````````````````````````````````````````````
subroutine manage_memory(dim_n_dims, dim_n_vars, dim_n_g_Atts,  &
     dim_max_var_ndims, dim_max_var_nAtts)

!+ Allocates or deallocates miscellaneous arrays with various dimensions

use univ_IO_mem_status_mod, only: illegal_alloc_stop
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

logical, optional, intent(IN) :: dim_n_dims, dim_n_vars, dim_n_g_Atts,  &
     dim_max_var_ndims, dim_max_var_nAtts

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'manage_memory'

integer :: a_err

!=== Executable statements:

if (present(dim_n_dims)) then
   if (dim_n_dims) then
      allocate(dim_name(n_dims), dimv(n_dims), stat=a_err) 
   else
      deallocate(dim_name, dimv, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_n_dims, procedure_name,  &
        'dim_n_dims')
end if

if (present(dim_n_vars)) then
   if (dim_n_vars) then
      allocate(var_name(n_vars), var_type(n_vars), var_ndims(n_vars),  &
           var_nAtts(n_vars), stat=a_err) 
   else
      deallocate(var_name, var_type, var_ndims, var_nAtts, stat=a_err) 
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_n_vars, procedure_name,  &
        'dim_n_vars')
end if

if (present(dim_n_g_Atts)) then
   if (dim_n_g_Atts) then
      allocate(g_Att(n_g_Atts), stat=a_err) 
   else
      deallocate(g_Att, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_n_g_Atts, procedure_name,  &
        'dim_n_g_Atts')
end if

if (present(dim_max_var_ndims)) then
   if (dim_max_var_ndims) then
      allocate(var_dimids(max_var_ndims, n_vars), stat=a_err) 
   else
      deallocate(var_dimids, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_max_var_ndims, procedure_name,  &
        'dim_max_var_ndims')
end if

if (present(dim_max_var_nAtts)) then
   if (dim_max_var_nAtts) then
      allocate(var_Att(max_var_nAtts, n_vars), stat=a_err) 
   else
      deallocate(var_Att, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_max_var_nAtts, procedure_name,  &
        'dim_max_var_nAtts')
end if

end subroutine manage_memory


!```````````````````````````````````````````````````````````````````
subroutine define_output_file(initialize, desc_str)

!+

use manage_NetCDF_classic_mod, only:  mNc_SHORT_type, mNc_define_md_Att_obj,  &
     mNc_CHAR_type

implicit none

!=== Argument declarations:

logical, intent(IN) :: initialize
character(len=*), intent(IN) :: desc_str

!== Local declarations:

integer :: i, ic, ia

!=== Executable statements:

! Total number of dimensions, variables, and global attributes:
n_dims = 2; n_vars = 1; n_g_Atts = 7

! Allocate and define "dimensions":
if (initialize) call manage_memory(dim_n_dims=.TRUE.)
i = 1; dim_name(i) = 'lon'; dimv(i) = n_d1_pts
i = 2; dim_name(i) = 'lat'; dimv(i) = n_d2_pts

! Allocate some variable- and global_attribute-related items:
if (initialize) call manage_memory(dim_n_vars=.TRUE., dim_n_g_Atts=.TRUE.)

! Define "global attributes":
i = 1;
tmp_mNc_str = 'native coordinates are IAU2000; lat values [degrees_N], '//  &
     'lon values [degrees_E]; MOLA dataset from NASA PDS; uninterpolated '//  &
     '1/64 data used poleward of 88 degrees latitude (to fill missing data)'
ic = len_trim(tmp_mNc_str)
call mNc_define_md_Att_obj(g_Att(i), 'dataset_misc_info', i, ic, tmp_mNc_str)
i = 2; tmp_mNc_str = 'this file was created 20.02.2012 by TIM'
ic = len_trim(tmp_mNc_str)
call mNc_define_md_Att_obj(g_Att(i), 'history', i, ic, tmp_mNc_str)
i = 3; call mNc_define_md_Att_obj(g_Att(i), 'lat_bounds_cell_center', i, 2,  &
     lat_bounds)
i = 4; call mNc_define_md_Att_obj(g_Att(i), 'lon_bounds_cell_center', i, 2,  &
     lon_bounds)
i = 5; call mNc_define_md_Att_obj(g_Att(i), 'lat_increment', i, 1, (/ dlat /))
i = 6; call mNc_define_md_Att_obj(g_Att(i), 'lon_increment', i, 1, (/ dlon /))
i = 7; call mNc_define_md_Att_obj(g_Att(i), 'lon_native_to_IAU2000', i, 1,  &
     (/ lon_native_to_IAU2000 /))

! Allocate variable-related items:
max_var_ndims = 2; max_var_nAtts = 4
if (initialize) call manage_memory(dim_max_var_ndims=.TRUE.,  &
     dim_max_var_nAtts=.TRUE.)

! Define "variables":

i = 1; var_name(i) = 'topo'; var_type(i) = mNc_SHORT_type
var_ndims(i) = 2; var_nAtts(i) = 4
var_dimids(1:var_ndims(i),i) = (/ 1, 2 /)
ia = 1; tmp_mNc_str = trim(desc_str); ic = len_trim(tmp_mNc_str)
call mNc_define_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
ia = 2; tmp_mNc_str = 'm'; ic = len_trim(tmp_mNc_str)
call mNc_define_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
ia = 3; call mNc_define_md_Att_obj(var_Att(ia,i), 'missing_value', ia, 1,  &
     (/ -20000_i2 /))
ia = 4; call mNc_define_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
     (/ -20000_i2 /))

end subroutine define_output_file


end program topo_Mg128_convert_to_nc

